@@ -1,43 +1,7 @@
-/****************************************************************************
-**
-** Copyright (C) 2017 BogDan Vatra <bogdan@kde.org>
-** Copyright (C) 2016 The Qt Company Ltd.
-** Copyright (C) 2016 Olivier Goffart <ogoffart@woboq.com>
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the Android port of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2017 BogDan Vatra <bogdan@kde.org>
+// Copyright (C) 2022 The Qt Company Ltd.
+// Copyright (C) 2016 Olivier Goffart <ogoffart@woboq.com>
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 package org.qtproject.qt.android;
 
@@ -50,6 +14,7 @@ import android.content.pm.PackageManager;
 import android.content.pm.PackageManager.NameNotFoundException;
 import android.content.res.AssetManager;
 import android.content.res.Configuration;
+import android.graphics.Color;
 import android.graphics.drawable.ColorDrawable;
 import android.graphics.drawable.Drawable;
 import android.graphics.Rect;
@@ -69,6 +34,7 @@ import android.view.animation.AlphaAnimation;
 import android.view.animation.Animation;
 import android.view.ContextMenu;
 import android.view.ContextMenu.ContextMenuInfo;
+import android.view.Display;
 import android.view.KeyCharacterMap;
 import android.view.KeyEvent;
 import android.view.Menu;
@@ -78,6 +44,8 @@ import android.view.Surface;
 import android.view.View;
 import android.view.ViewConfiguration;
 import android.view.ViewGroup;
+import android.view.Window;
+import android.view.WindowInsetsController;
 import android.view.WindowManager;
 import android.view.inputmethod.InputMethodManager;
 import android.view.ViewTreeObserver;
@@ -119,15 +87,11 @@ public class QtActivityDelegate
     private static final String ENVIRONMENT_VARIABLES_KEY = "environment.variables";
     private static final String APPLICATION_PARAMETERS_KEY = "application.parameters";
     private static final String STATIC_INIT_CLASSES_KEY = "static.init.classes";
-    private static final String NECESSITAS_API_LEVEL_KEY = "necessitas.api.level";
-    private static final String EXTRACT_STYLE_KEY = "extract.android.style";
-    private static final String EXTRACT_STYLE_MINIMAL_KEY = "extract.android.style.option";
 
     public static final int SYSTEM_UI_VISIBILITY_NORMAL = 0;
     public static final int SYSTEM_UI_VISIBILITY_FULLSCREEN = 1;
     public static final int SYSTEM_UI_VISIBILITY_TRANSLUCENT = 2;
 
-    private static String m_environmentVariables = null;
     private static String m_applicationParameters = null;
 
     private int m_currentRotation = -1; // undefined
@@ -158,6 +122,7 @@ public class QtActivityDelegate
     private CursorHandle m_leftSelectionHandle;
     private CursorHandle m_rightSelectionHandle;
     private EditPopupMenu m_editPopupMenu;
+    private boolean m_isPluginRunning = false;
 
     private QtAccessibilityDelegate m_accessibilityDelegate = null;
 
@@ -209,6 +174,11 @@ public class QtActivityDelegate
         }
     }
 
+    public boolean isKeyboardVisible()
+    {
+        return m_keyboardIsVisible;
+    }
+
     // input method hints - must be kept in sync with QTDIR/src/corelib/global/qnamespace.h
     private final int ImhHiddenText = 0x1;
     private final int ImhSensitiveData = 0x2;
@@ -260,7 +230,7 @@ public class QtActivityDelegate
         if (m_keyboardIsVisible == visibility)
             return false;
         m_keyboardIsVisible = visibility;
-        QtNative.keyboardVisibilityChanged(m_keyboardIsVisible);
+        QtNative.keyboardVisibilityUpdated(m_keyboardIsVisible);
 
         if (visibility == false)
             updateFullScreen(); // Hiding the keyboard clears the immersive mode, so we need to set it again.
@@ -343,7 +313,7 @@ public class QtActivityDelegate
             }
 
             if ((inputHints & ImhHiddenText) != 0)
-                inputType |= 0x10 /* TYPE_NUMBER_VARIATION_PASSWORD */;
+                inputType |= android.text.InputType.TYPE_NUMBER_VARIATION_PASSWORD;
         } else if ((inputHints & ImhDialableCharactersOnly) != 0) {
             inputType = android.text.InputType.TYPE_CLASS_PHONE;
         } else if ((inputHints & (ImhDate | ImhTime)) != 0) {
@@ -351,29 +321,32 @@ public class QtActivityDelegate
             if ((inputHints & (ImhDate | ImhTime)) != (ImhDate | ImhTime)) {
                 if ((inputHints & ImhDate) != 0)
                     inputType |= android.text.InputType.TYPE_DATETIME_VARIATION_DATE;
-                if ((inputHints & ImhTime) != 0)
+                else
                     inputType |= android.text.InputType.TYPE_DATETIME_VARIATION_TIME;
             } // else {  TYPE_DATETIME_VARIATION_NORMAL(0) }
         } else { // CLASS_TEXT
-            if ((inputHints & (ImhEmailCharactersOnly | ImhUrlCharactersOnly)) != 0) {
-                if ((inputHints & ImhUrlCharactersOnly) != 0) {
-                    inputType |= android.text.InputType.TYPE_TEXT_VARIATION_URI;
-
-                    if (enterKeyType == 0) // not explicitly overridden
-                        imeOptions = android.view.inputmethod.EditorInfo.IME_ACTION_GO;
-                } else if ((inputHints & ImhEmailCharactersOnly) != 0) {
-                    inputType |= android.text.InputType.TYPE_TEXT_VARIATION_EMAIL_ADDRESS;
-                }
-            } else if ((inputHints & ImhHiddenText) != 0) {
+            if ((inputHints & ImhHiddenText) != 0) {
                 inputType |= android.text.InputType.TYPE_TEXT_VARIATION_PASSWORD;
             } else if ((inputHints & ImhSensitiveData) != 0 ||
                 ((inputHints & ImhNoPredictiveText) != 0 &&
                   System.getenv("QT_ANDROID_ENABLE_WORKAROUND_TO_DISABLE_PREDICTIVE_TEXT") != null)) {
                 inputType |= android.text.InputType.TYPE_TEXT_VARIATION_VISIBLE_PASSWORD;
+            } else if ((inputHints & ImhUrlCharactersOnly) != 0) {
+                inputType |= android.text.InputType.TYPE_TEXT_VARIATION_URI;
+                if (enterKeyType == 0) // not explicitly overridden
+                    imeOptions = android.view.inputmethod.EditorInfo.IME_ACTION_GO;
+            } else if ((inputHints & ImhEmailCharactersOnly) != 0) {
+                inputType |= android.text.InputType.TYPE_TEXT_VARIATION_EMAIL_ADDRESS;
             }
 
-            if ((inputHints & ImhMultiLine) != 0)
+            if ((inputHints & ImhMultiLine) != 0) {
                 inputType |= android.text.InputType.TYPE_TEXT_FLAG_MULTI_LINE;
+                // Clear imeOptions for Multi-Line Type
+                // User should be able to insert new line in such case
+                imeOptions = android.view.inputmethod.EditorInfo.IME_ACTION_DONE;
+            }
+            if ((inputHints & (ImhNoPredictiveText | ImhSensitiveData | ImhHiddenText)) != 0)
+                inputType |= android.text.InputType.TYPE_TEXT_FLAG_NO_SUGGESTIONS;
 
             if ((inputHints & ImhUppercaseOnly) != 0) {
                 initialCapsMode |= android.text.TextUtils.CAP_MODE_CHARACTERS;
@@ -382,11 +355,6 @@ public class QtActivityDelegate
                 initialCapsMode |= android.text.TextUtils.CAP_MODE_SENTENCES;
                 inputType |= android.text.InputType.TYPE_TEXT_FLAG_CAP_SENTENCES;
             }
-
-            if ((inputHints & ImhNoPredictiveText) != 0 || (inputHints & ImhSensitiveData) != 0
-                || (inputHints & ImhHiddenText) != 0) {
-                inputType |= android.text.InputType.TYPE_TEXT_FLAG_NO_SUGGESTIONS;
-            }
         }
 
         if (enterKeyType == 0 && (inputHints & ImhMultiLine) != 0)
@@ -479,7 +447,7 @@ public class QtActivityDelegate
         });
     }
 
-    String getAppIconSize(Activity a)
+    int getAppIconSize(Activity a)
     {
         int size = a.getResources().getDimensionPixelSize(android.R.dimen.app_icon_size);
         if (size < 36 || size > 512) { // check size sanity
@@ -492,7 +460,8 @@ public class QtActivityDelegate
             if (size > 512)
                 size = 512;
         }
-        return "\tQT_ANDROID_APP_ICON_SIZE=" + size;
+
+        return size;
     }
 
     public void updateSelection(int selStart, int selEnd, int candidatesStart, int candidatesEnd)
@@ -509,6 +478,17 @@ public class QtActivityDelegate
     private static final int CursorHandleShowSelection  = 2;
     private static final int CursorHandleShowEdit       = 0x100;
 
+    public int getSelectHandleWidth()
+    {
+        int width = 0;
+        if (m_leftSelectionHandle != null && m_rightSelectionHandle != null) {
+            width = Math.max(m_leftSelectionHandle.width(), m_rightSelectionHandle.width());
+        } else if (m_cursorHandle != null) {
+            width = m_cursorHandle.width();
+        }
+        return width;
+    }
+
     /* called from the C++ code when the position of the cursor or selection handles needs to
        be adjusted.
        mode is one of QAndroidInputContext::CursorHandleShowMode
@@ -567,9 +547,7 @@ public class QtActivityDelegate
                 break;
         }
 
-        if (QtNative.hasClipboardText())
-            editButtons |= EditContextView.PASTE_BUTTON;
-        else
+        if (!QtNative.hasClipboardText())
             editButtons &= ~EditContextView.PASTE_BUTTON;
 
         if ((mode & CursorHandleShowEdit) == CursorHandleShowEdit && editButtons != 0) {
@@ -581,6 +559,100 @@ public class QtActivityDelegate
         }
     }
 
+    private final DisplayManager.DisplayListener displayListener = new DisplayManager.DisplayListener()
+    {
+        @Override
+        public void onDisplayAdded(int displayId) { }
+
+        private boolean isSimilarRotation(int r1, int r2)
+        {
+         return (r1 == r2)
+                || (r1 == Surface.ROTATION_0 && r2 == Surface.ROTATION_180)
+                || (r1 == Surface.ROTATION_180 && r2 == Surface.ROTATION_0)
+                || (r1 == Surface.ROTATION_90 && r2 == Surface.ROTATION_270)
+                || (r1 == Surface.ROTATION_270 && r2 == Surface.ROTATION_90);
+        }
+
+        @Override
+        public void onDisplayChanged(int displayId)
+        {
+            Display display = (Build.VERSION.SDK_INT < Build.VERSION_CODES.R)
+                 ? m_activity.getWindowManager().getDefaultDisplay()
+                 : m_activity.getDisplay();
+            m_currentRotation = display.getRotation();
+            m_layout.setActivityDisplayRotation(m_currentRotation);
+            // Process orientation change only if it comes after the size
+            // change, or if the screen is rotated by 180 degrees.
+            // Otherwise it will be processed in QtLayout.
+            if (isSimilarRotation(m_currentRotation, m_layout.displayRotation()))
+                QtNative.handleOrientationChanged(m_currentRotation, m_nativeOrientation);
+
+            float refreshRate = display.getRefreshRate();
+            QtNative.handleRefreshRateChanged(refreshRate);
+        }
+
+        @Override
+        public void onDisplayRemoved(int displayId) { }
+    };
+
+    public boolean updateActivity(Activity activity)
+    {
+        try {
+            // set new activity
+            loadActivity(activity);
+
+            // update the new activity content view to old layout
+            ViewGroup layoutParent = (ViewGroup)m_layout.getParent();
+            if (layoutParent != null)
+                layoutParent.removeView(m_layout);
+
+            m_activity.setContentView(m_layout);
+
+            // force c++ native activity object to update
+            return QtNative.updateNativeActivity();
+        } catch (Exception e) {
+            Log.w(QtNative.QtTAG, "Failed to update the activity.");
+            e.printStackTrace();
+            return false;
+        }
+    }
+
+    private void loadActivity(Activity activity)
+        throws NoSuchMethodException, PackageManager.NameNotFoundException
+    {
+        m_activity = activity;
+
+        QtNative.setActivity(m_activity, this);
+        setActionBarVisibility(false);
+
+        Class<?> activityClass = m_activity.getClass();
+        m_super_dispatchKeyEvent =
+                activityClass.getMethod("super_dispatchKeyEvent", KeyEvent.class);
+        m_super_onRestoreInstanceState =
+                activityClass.getMethod("super_onRestoreInstanceState", Bundle.class);
+        m_super_onRetainNonConfigurationInstance =
+                activityClass.getMethod("super_onRetainNonConfigurationInstance");
+        m_super_onSaveInstanceState =
+                activityClass.getMethod("super_onSaveInstanceState", Bundle.class);
+        m_super_onKeyDown =
+                activityClass.getMethod("super_onKeyDown", Integer.TYPE, KeyEvent.class);
+        m_super_onKeyUp =
+                activityClass.getMethod("super_onKeyUp", Integer.TYPE, KeyEvent.class);
+        m_super_onConfigurationChanged =
+                activityClass.getMethod("super_onConfigurationChanged", Configuration.class);
+        m_super_onActivityResult =
+                activityClass.getMethod("super_onActivityResult", Integer.TYPE, Integer.TYPE, Intent.class);
+        m_super_onWindowFocusChanged =
+                activityClass.getMethod("super_onWindowFocusChanged", Boolean.TYPE);
+        m_super_dispatchGenericMotionEvent =
+                activityClass.getMethod("super_dispatchGenericMotionEvent", MotionEvent.class);
+
+        m_softInputMode = m_activity.getPackageManager().getActivityInfo(m_activity.getComponentName(), 0).softInputMode;
+
+        DisplayManager displayManager = (DisplayManager)m_activity.getSystemService(Context.DISPLAY_SERVICE);
+        displayManager.registerDisplayListener(displayListener, null);
+    }
+
     public boolean loadApplication(Activity activity, ClassLoader classLoader, Bundle loaderParams)
     {
         /// check parameters integrity
@@ -590,10 +662,14 @@ public class QtActivityDelegate
             return false;
         }
 
-        m_activity = activity;
-        setActionBarVisibility(false);
-        QtNative.setActivity(m_activity, this);
-        QtNative.setClassLoader(classLoader);
+        try {
+            loadActivity(activity);
+            QtNative.setClassLoader(classLoader);
+        } catch (Exception e) {
+            e.printStackTrace();
+            return false;
+        }
+
         if (loaderParams.containsKey(STATIC_INIT_CLASSES_KEY)) {
             for (String className: Objects.requireNonNull(loaderParams.getStringArray(STATIC_INIT_CLASSES_KEY))) {
                 if (className.length() == 0)
@@ -632,80 +708,25 @@ public class QtActivityDelegate
             libraries.remove(libraries.size() - 1);
         }
 
-        if (loaderParams.containsKey(EXTRACT_STYLE_KEY)) {
-            String path = loaderParams.getString(EXTRACT_STYLE_KEY);
-            new ExtractStyle(m_activity, path, loaderParams.containsKey(EXTRACT_STYLE_MINIMAL_KEY) &&
-                                               loaderParams.getBoolean(EXTRACT_STYLE_MINIMAL_KEY));
-        }
+        ExtractStyle.setup(loaderParams);
+        ExtractStyle.runIfNeeded(m_activity, isUiModeDark(m_activity.getResources().getConfiguration()));
 
-        try {
-            m_super_dispatchKeyEvent = m_activity.getClass().getMethod("super_dispatchKeyEvent", KeyEvent.class);
-            m_super_onRestoreInstanceState = m_activity.getClass().getMethod("super_onRestoreInstanceState", Bundle.class);
-            m_super_onRetainNonConfigurationInstance = m_activity.getClass().getMethod("super_onRetainNonConfigurationInstance");
-            m_super_onSaveInstanceState = m_activity.getClass().getMethod("super_onSaveInstanceState", Bundle.class);
-            m_super_onKeyDown = m_activity.getClass().getMethod("super_onKeyDown", Integer.TYPE, KeyEvent.class);
-            m_super_onKeyUp = m_activity.getClass().getMethod("super_onKeyUp", Integer.TYPE, KeyEvent.class);
-            m_super_onConfigurationChanged = m_activity.getClass().getMethod("super_onConfigurationChanged", Configuration.class);
-            m_super_onActivityResult = m_activity.getClass().getMethod("super_onActivityResult", Integer.TYPE, Integer.TYPE, Intent.class);
-            m_super_onWindowFocusChanged = m_activity.getClass().getMethod("super_onWindowFocusChanged", Boolean.TYPE);
-            m_super_dispatchGenericMotionEvent = m_activity.getClass().getMethod("super_dispatchGenericMotionEvent", MotionEvent.class);
-        } catch (Exception e) {
-            e.printStackTrace();
-            return false;
-        }
-
-        int necessitasApiLevel = 1;
-        if (loaderParams.containsKey(NECESSITAS_API_LEVEL_KEY))
-            necessitasApiLevel = loaderParams.getInt(NECESSITAS_API_LEVEL_KEY);
-
-        m_environmentVariables = loaderParams.getString(ENVIRONMENT_VARIABLES_KEY);
-        String additionalEnvironmentVariables = "QT_ANDROID_FONTS_MONOSPACE=Droid Sans Mono;Droid Sans;Droid Sans Fallback"
-                                              + "\tQT_ANDROID_FONTS_SERIF=Droid Serif"
-                                              + "\tNECESSITAS_API_LEVEL=" + necessitasApiLevel
-                                              + "\tHOME=" + m_activity.getFilesDir().getAbsolutePath()
-                                              + "\tTMPDIR=" + m_activity.getFilesDir().getAbsolutePath();
-
-        additionalEnvironmentVariables += "\tQT_ANDROID_FONTS=Roboto;Droid Sans;Droid Sans Fallback";
-
-        additionalEnvironmentVariables += getAppIconSize(activity);
-
-        if (m_environmentVariables != null && m_environmentVariables.length() > 0)
-            m_environmentVariables = additionalEnvironmentVariables + "\t" + m_environmentVariables;
-        else
-            m_environmentVariables = additionalEnvironmentVariables;
+        QtNative.setEnvironmentVariables(loaderParams.getString(ENVIRONMENT_VARIABLES_KEY));
+        QtNative.setEnvironmentVariable("QT_ANDROID_FONTS_MONOSPACE",
+                                        "Droid Sans Mono;Droid Sans;Droid Sans Fallback");
+        QtNative.setEnvironmentVariable("QT_ANDROID_FONTS_SERIF", "Droid Serif");
+        QtNative.setEnvironmentVariable("HOME", m_activity.getFilesDir().getAbsolutePath());
+        QtNative.setEnvironmentVariable("TMPDIR", m_activity.getFilesDir().getAbsolutePath());
+        QtNative.setEnvironmentVariable("QT_ANDROID_FONTS",
+                                        "Roboto;Droid Sans;Droid Sans Fallback");
+        QtNative.setEnvironmentVariable("QT_ANDROID_APP_ICON_SIZE",
+                                        String.valueOf(getAppIconSize(activity)));
 
         if (loaderParams.containsKey(APPLICATION_PARAMETERS_KEY))
             m_applicationParameters = loaderParams.getString(APPLICATION_PARAMETERS_KEY);
         else
             m_applicationParameters = "";
 
-        try {
-            m_softInputMode = m_activity.getPackageManager().getActivityInfo(m_activity.getComponentName(), 0).softInputMode;
-        } catch (Exception e) {
-            e.printStackTrace();
-        }
-
-        DisplayManager.DisplayListener displayListener = new DisplayManager.DisplayListener() {
-            @Override
-            public void onDisplayAdded(int displayId) { }
-
-            @Override
-            public void onDisplayChanged(int displayId) {
-                m_currentRotation = m_activity.getWindowManager().getDefaultDisplay().getRotation();
-                QtNative.handleOrientationChanged(m_currentRotation, m_nativeOrientation);
-            }
-
-            @Override
-            public void onDisplayRemoved(int displayId) { }
-        };
-
-        try {
-            DisplayManager displayManager = (DisplayManager) m_activity.getSystemService(Context.DISPLAY_SERVICE);
-            displayManager.registerDisplayListener(displayListener, null);
-        } catch (Exception e) {
-            e.printStackTrace();
-        }
-
         m_mainLib = QtNative.loadMainLibrary(m_mainLib, nativeLibsDir);
         return m_mainLib != null;
     }
@@ -724,7 +745,9 @@ public class QtActivityDelegate
 
                     if (extras.containsKey("extraenvvars")) {
                         try {
-                            m_environmentVariables += "\t" + new String(Base64.decode(extras.getString("extraenvvars"), Base64.DEFAULT), "UTF-8");
+                            QtNative.setEnvironmentVariables(new String(
+                                    Base64.decode(extras.getString("extraenvvars"), Base64.DEFAULT),
+                                    "UTF-8"));
                         } catch (Exception e) {
                             e.printStackTrace();
                         }
@@ -769,7 +792,7 @@ public class QtActivityDelegate
                 @Override
                 public void run() {
                     try {
-                        QtNative.startApplication(m_applicationParameters, m_environmentVariables, m_mainLib);
+                        QtNative.startApplication(m_applicationParameters, m_mainLib);
                         m_started = true;
                     } catch (Exception e) {
                         e.printStackTrace();
@@ -794,7 +817,7 @@ public class QtActivityDelegate
                 m_splashScreenSticky = info.metaData.containsKey("android.app.splash_screen_sticky") && info.metaData.getBoolean("android.app.splash_screen_sticky");
                 int id = info.metaData.getInt(splashScreenKey);
                 m_splashScreen = new ImageView(m_activity);
-                m_splashScreen.setImageDrawable(m_activity.getResources().getDrawable(id));
+                m_splashScreen.setImageDrawable(m_activity.getResources().getDrawable(id, m_activity.getTheme()));
                 m_splashScreen.setScaleType(ImageView.ScaleType.FIT_XY);
                 m_splashScreen.setLayoutParams(new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
                 m_layout.addView(m_splashScreen);
@@ -820,9 +843,17 @@ public class QtActivityDelegate
         else
             m_nativeOrientation = Configuration.ORIENTATION_PORTRAIT;
 
+        m_layout.setNativeOrientation(m_nativeOrientation);
         QtNative.handleOrientationChanged(rotation, m_nativeOrientation);
         m_currentRotation = rotation;
 
+        handleUiModeChange(m_activity.getResources().getConfiguration().uiMode & Configuration.UI_MODE_NIGHT_MASK);
+
+        float refreshRate = (Build.VERSION.SDK_INT < Build.VERSION_CODES.R)
+                ? m_activity.getWindowManager().getDefaultDisplay().getRefreshRate()
+                : m_activity.getDisplay().getRefreshRate();
+        QtNative.handleRefreshRateChanged(refreshRate);
+
         m_layout.getViewTreeObserver().addOnPreDrawListener(new ViewTreeObserver.OnPreDrawListener() {
             @Override
             public boolean onPreDraw() {
@@ -834,6 +865,10 @@ public class QtActivityDelegate
                 DisplayMetrics metrics = new DisplayMetrics();
                 m_activity.getWindowManager().getDefaultDisplay().getMetrics(metrics);
                 final int kbHeight = metrics.heightPixels - r.bottom;
+                if (kbHeight < 0) {
+                    setKeyboardVisibility(false, System.nanoTime());
+                    return true;
+                }
                 final int[] location = new int[2];
                 m_layout.getLocationOnScreen(location);
                 QtNative.keyboardGeometryChanged(location[0], r.bottom - location[1],
@@ -878,18 +913,18 @@ public class QtActivityDelegate
         m_splashScreen.startAnimation(fadeOut);
     }
 
-    public void notifyAccessibilityLocationChange()
+    public void notifyAccessibilityLocationChange(int viewId)
     {
         if (m_accessibilityDelegate == null)
             return;
-        m_accessibilityDelegate.notifyLocationChange();
+        m_accessibilityDelegate.notifyLocationChange(viewId);
     }
 
-    public void notifyObjectHide(int viewId)
+    public void notifyObjectHide(int viewId, int parentId)
     {
         if (m_accessibilityDelegate == null)
             return;
-        m_accessibilityDelegate.notifyObjectHide(viewId);
+        m_accessibilityDelegate.notifyObjectHide(viewId, parentId);
     }
 
     public void notifyObjectFocus(int viewId)
@@ -899,6 +934,26 @@ public class QtActivityDelegate
         m_accessibilityDelegate.notifyObjectFocus(viewId);
     }
 
+    public void notifyValueChanged(int viewId, String value)
+    {
+        if (m_accessibilityDelegate == null)
+            return;
+        m_accessibilityDelegate.notifyValueChanged(viewId, value);
+    }
+
+    public void notifyScrolledEvent(int viewId)
+    {
+        if (m_accessibilityDelegate == null)
+            return;
+        m_accessibilityDelegate.notifyScrolledEvent(viewId);
+    }
+
+
+    public void notifyQtAndroidPluginRunning(boolean running)
+    {
+        m_isPluginRunning = running;
+    }
+
     public void initializeAccessibility()
     {
         m_accessibilityDelegate = new QtAccessibilityDelegate(m_activity, m_layout, this);
@@ -914,6 +969,38 @@ public class QtActivityDelegate
             updateFullScreen();
     }
 
+    boolean isUiModeDark(Configuration config)
+    {
+        return (config.uiMode & Configuration.UI_MODE_NIGHT_MASK) == Configuration.UI_MODE_NIGHT_YES;
+    }
+
+    private void handleUiModeChange(int uiMode)
+    {
+        // QTBUG-108365
+        if (Build.VERSION.SDK_INT >= 30) {
+            // Since 29 version we are using Theme_DeviceDefault_DayNight
+            Window window = m_activity.getWindow();
+            WindowInsetsController controller = window.getInsetsController();
+            if (controller != null) {
+                // set APPEARANCE_LIGHT_STATUS_BARS if needed
+                int appearanceLight = Color.luminance(window.getStatusBarColor()) > 0.5 ?
+                        WindowInsetsController.APPEARANCE_LIGHT_STATUS_BARS : 0;
+                controller.setSystemBarsAppearance(appearanceLight,
+                    WindowInsetsController.APPEARANCE_LIGHT_STATUS_BARS);
+            }
+        }
+        switch (uiMode) {
+            case Configuration.UI_MODE_NIGHT_NO:
+                ExtractStyle.runIfNeeded(m_activity, false);
+                QtNative.handleUiDarkModeChanged(0);
+                break;
+            case Configuration.UI_MODE_NIGHT_YES:
+                ExtractStyle.runIfNeeded(m_activity, true);
+                QtNative.handleUiDarkModeChanged(1);
+                break;
+        }
+    }
+
     public void onConfigurationChanged(Configuration configuration)
     {
         try {
@@ -921,6 +1008,7 @@ public class QtActivityDelegate
         } catch (Exception e) {
             e.printStackTrace();
         }
+        handleUiModeChange(configuration.uiMode & Configuration.UI_MODE_NIGHT_MASK);
     }
 
     public void onDestroy()
@@ -1006,7 +1094,7 @@ public class QtActivityDelegate
 
     public boolean onKeyDown(int keyCode, KeyEvent event)
     {
-        if (!m_started)
+        if (!m_started || !m_isPluginRunning)
             return false;
 
         m_metaState = MetaKeyKeyListener.handleKeyDown(m_metaState, keyCode, event);
@@ -1040,7 +1128,7 @@ public class QtActivityDelegate
 
     public boolean onKeyUp(int keyCode, KeyEvent event)
     {
-        if (!m_started)
+        if (!m_started || !m_isPluginRunning)
             return false;
 
         if ((keyCode == KeyEvent.KEYCODE_VOLUME_UP
@@ -1211,7 +1299,7 @@ public class QtActivityDelegate
             if (attr.type >= TypedValue.TYPE_FIRST_COLOR_INT && attr.type <= TypedValue.TYPE_LAST_COLOR_INT) {
                 m_activity.getWindow().setBackgroundDrawable(new ColorDrawable(attr.data));
             } else {
-                m_activity.getWindow().setBackgroundDrawable(m_activity.getResources().getDrawable(attr.resourceId));
+                m_activity.getWindow().setBackgroundDrawable(m_activity.getResources().getDrawable(attr.resourceId, m_activity.getTheme()));
             }
             if (m_dummyView != null) {
                 m_layout.removeView(m_dummyView);
