@@ -1,51 +1,19 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtCore module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include <qjsonobject.h>
 #include <qjsonvalue.h>
 #include <qjsonarray.h>
+#include <qjsondocument.h>
 #include <qstringlist.h>
+#include <qcborarray.h>
 #include <qvariant.h>
 #include <qdebug.h>
 
+#include <private/qcborvalue_p.h>
+#include <private/qjson_p.h>
+
 #include "qjsonwriter_p.h"
-#include "qjson_p.h"
 
 QT_BEGIN_NAMESPACE
 
@@ -54,6 +22,7 @@ QT_BEGIN_NAMESPACE
     \inmodule QtCore
     \ingroup json
     \ingroup shared
+    \ingroup qtserialization
     \reentrant
     \since 5.0
 
@@ -89,7 +58,7 @@ QT_BEGIN_NAMESPACE
 /*!
     \typedef QJsonArray::size_type
 
-    Typedef for int. Provided for STL compatibility.
+    Typedef for qsizetype. Provided for STL compatibility.
 */
 
 /*!
@@ -101,7 +70,7 @@ QT_BEGIN_NAMESPACE
 /*!
     \typedef QJsonArray::difference_type
 
-    Typedef for int. Provided for STL compatibility.
+    Typedef for qsizetype. Provided for STL compatibility.
 */
 
 /*!
@@ -131,10 +100,7 @@ QT_BEGIN_NAMESPACE
 /*!
     Creates an empty array.
  */
-QJsonArray::QJsonArray()
-    : d(nullptr), a(nullptr)
-{
-}
+QJsonArray::QJsonArray() = default;
 
 /*!
     \fn QJsonArray::QJsonArray(std::initializer_list<QJsonValue> args)
@@ -151,34 +117,20 @@ QJsonArray::QJsonArray()
 /*!
     \internal
  */
-QJsonArray::QJsonArray(QJsonPrivate::Data *data, QJsonPrivate::Array *array)
-    : d(data), a(array)
+QJsonArray::QJsonArray(QCborContainerPrivate *array)
+    : a(array)
 {
-    Q_ASSERT(data);
-    Q_ASSERT(array);
-    d->ref.ref();
-}
-
-/*!
-    This method replaces part of QJsonArray(std::initializer_list<QJsonValue> args) .
-    The constructor needs to be inline, but we do not want to leak implementation details
-    of this class.
-    \note this method is called for an uninitialized object
-    \internal
- */
-void QJsonArray::initialize()
-{
-    d = nullptr;
-    a = nullptr;
 }
 
 /*!
     Deletes the array.
  */
-QJsonArray::~QJsonArray()
+QJsonArray::~QJsonArray() = default;
+
+QJsonArray::QJsonArray(std::initializer_list<QJsonValue> args)
 {
-    if (d && !d->ref.deref())
-        delete d;
+    for (const auto & arg : args)
+        append(arg);
 }
 
 /*!
@@ -189,10 +141,13 @@ QJsonArray::~QJsonArray()
  */
 QJsonArray::QJsonArray(const QJsonArray &other)
 {
-    d = other.d;
     a = other.a;
-    if (d)
-        d->ref.ref();
+}
+
+QJsonArray::QJsonArray(QJsonArray &&other) noexcept
+    : a(other.a)
+{
+    other.a = nullptr;
 }
 
 /*!
@@ -200,15 +155,7 @@ QJsonArray::QJsonArray(const QJsonArray &other)
  */
 QJsonArray &QJsonArray::operator =(const QJsonArray &other)
 {
-    if (d != other.d) {
-        if (d && !d->ref.deref())
-            delete d;
-        d = other.d;
-        if (d)
-            d->ref.ref();
-    }
     a = other.a;
-
     return *this;
 }
 
@@ -278,52 +225,14 @@ QJsonArray QJsonArray::fromStringList(const QStringList &list)
 
     The QVariant values in \a list will be converted to JSON values.
 
+    \note Conversion from \l QVariant is not completely lossless. Please see
+    the documentation in QJsonValue::fromVariant() for more information.
+
     \sa toVariantList(), QJsonValue::fromVariant()
  */
 QJsonArray QJsonArray::fromVariantList(const QVariantList &list)
 {
-    QJsonArray array;
-    if (list.isEmpty())
-        return array;
-
-    array.detach2(1024);
-
-    QVector<QJsonPrivate::Value> values;
-    values.resize(list.size());
-    QJsonPrivate::Value *valueData = values.data();
-    uint currentOffset = sizeof(QJsonPrivate::Base);
-
-    for (int i = 0; i < list.size(); ++i) {
-        QJsonValue val = QJsonValue::fromVariant(list.at(i));
-
-        bool latinOrIntValue;
-        int valueSize = QJsonPrivate::Value::requiredStorage(val, &latinOrIntValue);
-
-        if (!array.detach2(valueSize))
-            return QJsonArray();
-
-        QJsonPrivate::Value *v = valueData + i;
-        v->type = (val.t == QJsonValue::Undefined ? QJsonValue::Null : val.t);
-        v->latinOrIntValue = latinOrIntValue;
-        v->latinKey = false;
-        v->value = QJsonPrivate::Value::valueToStore(val, currentOffset);
-        if (valueSize)
-            QJsonPrivate::Value::copyData(val, (char *)array.a + currentOffset, latinOrIntValue);
-
-        currentOffset += valueSize;
-        array.a->size = currentOffset;
-    }
-
-    // write table
-    array.a->tableOffset = currentOffset;
-    if (!array.detach2(sizeof(QJsonPrivate::offset)*values.size()))
-        return QJsonArray();
-    memcpy(static_cast<void *>(array.a->table()),
-           static_cast<const void *>(values.constData()), values.size()*sizeof(uint));
-    array.a->length = values.size();
-    array.a->size = currentOffset + sizeof(QJsonPrivate::offset)*values.size();
-
-    return array;
+    return QJsonPrivate::Variant::toJsonArray(list);
 }
 
 /*!
@@ -333,26 +242,16 @@ QJsonArray QJsonArray::fromVariantList(const QVariantList &list)
  */
 QVariantList QJsonArray::toVariantList() const
 {
-    QVariantList list;
-
-    if (a) {
-        list.reserve(a->length);
-        for (int i = 0; i < (int)a->length; ++i)
-            list.append(QJsonValue(d, a, a->at(i)).toVariant());
-    }
-    return list;
+    return QCborArray::fromJsonArray(*this).toVariantList();
 }
 
 
 /*!
     Returns the number of values stored in the array.
  */
-int QJsonArray::size() const
+qsizetype QJsonArray::size() const
 {
-    if (!d)
-        return 0;
-
-    return (int)a->length;
+    return a ? a->elements.size() : 0;
 }
 
 /*!
@@ -370,10 +269,7 @@ int QJsonArray::size() const
  */
 bool QJsonArray::isEmpty() const
 {
-    if (!d)
-        return true;
-
-    return !a->length;
+    return a == nullptr || a->elements.isEmpty();
 }
 
 /*!
@@ -382,12 +278,12 @@ bool QJsonArray::isEmpty() const
     The returned QJsonValue is \c Undefined, if \a i is out of bounds.
 
  */
-QJsonValue QJsonArray::at(int i) const
+QJsonValue QJsonArray::at(qsizetype i) const
 {
-    if (!a || i < 0 || i >= (int)a->length)
+    if (!a || i < 0 || i >= a->elements.size())
         return QJsonValue(QJsonValue::Undefined);
 
-    return QJsonValue(d, a, a->at(i));
+    return QJsonPrivate::Value::fromTrustedCbor(a->valueAt(i));
 }
 
 /*!
@@ -411,7 +307,7 @@ QJsonValue QJsonArray::first() const
  */
 QJsonValue QJsonArray::last() const
 {
-    return at(a ? (a->length - 1) : 0);
+    return at(a ? (a->elements.size() - 1) : 0);
 }
 
 /*!
@@ -433,7 +329,7 @@ void QJsonArray::prepend(const QJsonValue &value)
  */
 void QJsonArray::append(const QJsonValue &value)
 {
-    insert(a ? (int)a->length : 0, value);
+    insert(a ? a->elements.size() : 0, value);
 }
 
 /*!
@@ -442,16 +338,12 @@ void QJsonArray::append(const QJsonValue &value)
 
     \sa insert(), replace()
  */
-void QJsonArray::removeAt(int i)
+void QJsonArray::removeAt(qsizetype i)
 {
-    if (!a || i < 0 || i >= (int)a->length)
+    if (!a || i < 0 || i >= a->elements.size())
         return;
-
-    detach2();
-    a->removeItems(i, 1);
-    ++d->compactionCounter;
-    if (d->compactionCounter > 32u && d->compactionCounter >= unsigned(a->length) / 2u)
-        compact();
+    detach();
+    a->removeAt(i);
 }
 
 /*! \fn void QJsonArray::removeFirst()
@@ -482,13 +374,14 @@ void QJsonArray::removeAt(int i)
 
     \sa removeAt()
  */
-QJsonValue QJsonArray::takeAt(int i)
+QJsonValue QJsonArray::takeAt(qsizetype i)
 {
-    if (!a || i < 0 || i >= (int)a->length)
+    if (!a || i < 0 || i >= a->elements.size())
         return QJsonValue(QJsonValue::Undefined);
 
-    QJsonValue v(d, a, a->at(i));
-    removeAt(i); // detaches
+    detach();
+    const QJsonValue v = QJsonPrivate::Value::fromTrustedCbor(a->extractAt(i));
+    a->removeAt(i);
     return v;
 }
 
@@ -499,31 +392,16 @@ QJsonValue QJsonArray::takeAt(int i)
 
     \sa append(), prepend(), replace(), removeAt()
  */
-void QJsonArray::insert(int i, const QJsonValue &value)
+void QJsonArray::insert(qsizetype i, const QJsonValue &value)
 {
-    Q_ASSERT (i >= 0 && i <= (a ? (int)a->length : 0));
-    QJsonValue val = value;
+    if (a)
+        detach(a->elements.size() + 1);
+    else
+        a = new QCborContainerPrivate;
 
-    bool compressed;
-    int valueSize = QJsonPrivate::Value::requiredStorage(val, &compressed);
-
-    if (!detach2(valueSize + sizeof(QJsonPrivate::Value)))
-        return;
-
-    if (!a->length)
-        a->tableOffset = sizeof(QJsonPrivate::Array);
-
-    int valueOffset = a->reserveSpace(valueSize, i, 1, false);
-    if (!valueOffset)
-        return;
-
-    QJsonPrivate::Value &v = (*a)[i];
-    v.type = (val.t == QJsonValue::Undefined ? QJsonValue::Null : val.t);
-    v.latinOrIntValue = compressed;
-    v.latinKey = false;
-    v.value = QJsonPrivate::Value::valueToStore(val, valueOffset);
-    if (valueSize)
-        QJsonPrivate::Value::copyData(val, (char *)a + valueOffset, compressed);
+    Q_ASSERT (i >= 0 && i <= a->elements.size());
+    a->insertAt(i, value.type() == QJsonValue::Undefined ? QCborValue(nullptr)
+                                                         : QCborValue::fromJsonValue(value));
 }
 
 /*!
@@ -550,35 +428,11 @@ void QJsonArray::insert(int i, const QJsonValue &value)
 
     \sa operator[](), removeAt()
  */
-void QJsonArray::replace(int i, const QJsonValue &value)
+void QJsonArray::replace(qsizetype i, const QJsonValue &value)
 {
-    Q_ASSERT (a && i >= 0 && i < (int)(a->length));
-    QJsonValue val = value;
-
-    bool compressed;
-    int valueSize = QJsonPrivate::Value::requiredStorage(val, &compressed);
-
-    if (!detach2(valueSize))
-        return;
-
-    if (!a->length)
-        a->tableOffset = sizeof(QJsonPrivate::Array);
-
-    int valueOffset = a->reserveSpace(valueSize, i, 1, true);
-    if (!valueOffset)
-        return;
-
-    QJsonPrivate::Value &v = (*a)[i];
-    v.type = (val.t == QJsonValue::Undefined ? QJsonValue::Null : val.t);
-    v.latinOrIntValue = compressed;
-    v.latinKey = false;
-    v.value = QJsonPrivate::Value::valueToStore(val, valueOffset);
-    if (valueSize)
-        QJsonPrivate::Value::copyData(val, (char *)a + valueOffset, compressed);
-
-    ++d->compactionCounter;
-    if (d->compactionCounter > 32u && d->compactionCounter >= unsigned(a->length) / 2u)
-        compact();
+    Q_ASSERT (a && i >= 0 && i < a->elements.size());
+    detach();
+    a->replaceAt(i, QCborValue::fromJsonValue(value));
 }
 
 /*!
@@ -588,7 +442,7 @@ void QJsonArray::replace(int i, const QJsonValue &value)
  */
 bool QJsonArray::contains(const QJsonValue &value) const
 {
-    for (int i = 0; i < size(); i++) {
+    for (qsizetype i = 0; i < size(); i++) {
         if (at(i) == value)
             return true;
     }
@@ -608,9 +462,9 @@ bool QJsonArray::contains(const QJsonValue &value) const
 
     \sa at()
  */
-QJsonValueRef QJsonArray::operator [](int i)
+QJsonValueRef QJsonArray::operator [](qsizetype i)
 {
-    Q_ASSERT(a && i >= 0 && i < (int)a->length);
+    Q_ASSERT(a && i >= 0 && i < a->elements.size());
     return QJsonValueRef(this, i);
 }
 
@@ -619,7 +473,7 @@ QJsonValueRef QJsonArray::operator [](int i)
 
     Same as at().
  */
-QJsonValue QJsonArray::operator[](int i) const
+QJsonValue QJsonArray::operator[](qsizetype i) const
 {
     return at(i);
 }
@@ -633,14 +487,14 @@ bool QJsonArray::operator==(const QJsonArray &other) const
         return true;
 
     if (!a)
-        return !other.a->length;
+        return !other.a->elements.size();
     if (!other.a)
-        return !a->length;
-    if (a->length != other.a->length)
+        return !a->elements.size();
+    if (a->elements.size() != other.a->elements.size())
         return false;
 
-    for (int i = 0; i < (int)a->length; ++i) {
-        if (QJsonValue(d, a, a->at(i)) != QJsonValue(other.d, other.a, other.a->at(i)))
+    for (qsizetype i = 0; i < a->elements.size(); ++i) {
+        if (a->valueAt(i) != other.a->valueAt(i))
             return false;
     }
     return true;
@@ -810,7 +664,7 @@ bool QJsonArray::operator!=(const QJsonArray &other) const
     \sa QJsonArray::begin(), QJsonArray::end()
 */
 
-/*! \fn QJsonArray::iterator::iterator(QJsonArray *array, int index)
+/*! \fn QJsonArray::iterator::iterator(QJsonArray *array, qsizetype index)
     \internal
 */
 
@@ -834,7 +688,7 @@ bool QJsonArray::operator!=(const QJsonArray &other) const
     Returns a pointer to a modifiable reference to the current item.
 */
 
-/*! \fn QJsonValueRef QJsonArray::iterator::operator[](int j) const
+/*! \fn QJsonValueRef QJsonArray::iterator::operator[](qsizetype j) const
 
     Returns a modifiable reference to the item at offset \a j from the
     item pointed to by this iterator (the item at position \c{*this + j}).
@@ -941,7 +795,7 @@ bool QJsonArray::operator!=(const QJsonArray &other) const
     current and returns an iterator to the previously current item.
 */
 
-/*! \fn QJsonArray::iterator &QJsonArray::iterator::operator+=(int j)
+/*! \fn QJsonArray::iterator &QJsonArray::iterator::operator+=(qsizetype j)
 
     Advances the iterator by \a j items. If \a j is negative, the
     iterator goes backward.
@@ -949,7 +803,7 @@ bool QJsonArray::operator!=(const QJsonArray &other) const
     \sa operator-=(), operator+()
 */
 
-/*! \fn QJsonArray::iterator &QJsonArray::iterator::operator-=(int j)
+/*! \fn QJsonArray::iterator &QJsonArray::iterator::operator-=(qsizetype j)
 
     Makes the iterator go back by \a j items. If \a j is negative,
     the iterator goes forward.
@@ -957,7 +811,7 @@ bool QJsonArray::operator!=(const QJsonArray &other) const
     \sa operator+=(), operator-()
 */
 
-/*! \fn QJsonArray::iterator QJsonArray::iterator::operator+(int j) const
+/*! \fn QJsonArray::iterator QJsonArray::iterator::operator+(qsizetype j) const
 
     Returns an iterator to the item at \a j positions forward from
     this iterator. If \a j is negative, the iterator goes backward.
@@ -965,7 +819,7 @@ bool QJsonArray::operator!=(const QJsonArray &other) const
     \sa operator-(), operator+=()
 */
 
-/*! \fn QJsonArray::iterator QJsonArray::iterator::operator-(int j) const
+/*! \fn QJsonArray::iterator QJsonArray::iterator::operator-(qsizetype j) const
 
     Returns an iterator to the item at \a j positions backward from
     this iterator. If \a j is negative, the iterator goes forward.
@@ -973,7 +827,7 @@ bool QJsonArray::operator!=(const QJsonArray &other) const
     \sa operator+(), operator-=()
 */
 
-/*! \fn int QJsonArray::iterator::operator-(iterator other) const
+/*! \fn qsizetype QJsonArray::iterator::operator-(iterator other) const
 
     Returns the number of items between the item pointed to by \a
     other and the item pointed to by this iterator.
@@ -1019,7 +873,7 @@ bool QJsonArray::operator!=(const QJsonArray &other) const
     \sa QJsonArray::constBegin(), QJsonArray::constEnd()
 */
 
-/*! \fn QJsonArray::const_iterator::const_iterator(const QJsonArray *array, int index)
+/*! \fn QJsonArray::const_iterator::const_iterator(const QJsonArray *array, qsizetype index)
     \internal
 */
 
@@ -1049,27 +903,22 @@ bool QJsonArray::operator!=(const QJsonArray &other) const
     \internal
 */
 
-/*! \fn QJsonArray::const_iterator::const_iterator(const const_iterator &other)
-
-    Constructs a copy of \a other.
-*/
-
 /*! \fn QJsonArray::const_iterator::const_iterator(const iterator &other)
 
     Constructs a copy of \a other.
 */
 
-/*! \fn QJsonValue QJsonArray::const_iterator::operator*() const
+/*! \fn const QJsonValueRef QJsonArray::const_iterator::operator*() const
 
     Returns the current item.
 */
 
-/*! \fn QJsonValue *QJsonArray::const_iterator::operator->() const
+/*! \fn const QJsonValueRef *QJsonArray::const_iterator::operator->() const
 
     Returns a pointer to the current item.
 */
 
-/*! \fn QJsonValue QJsonArray::const_iterator::operator[](int j) const
+/*! \fn QJsonValue QJsonArray::const_iterator::operator[](qsizetype j) const
 
     Returns the item at offset \a j from the item pointed to by this iterator (the item at
     position \c{*this + j}).
@@ -1162,7 +1011,7 @@ bool QJsonArray::operator!=(const QJsonArray &other) const
     current and returns an iterator to the previously current item.
 */
 
-/*! \fn QJsonArray::const_iterator &QJsonArray::const_iterator::operator+=(int j)
+/*! \fn QJsonArray::const_iterator &QJsonArray::const_iterator::operator+=(qsizetype j)
 
     Advances the iterator by \a j items. If \a j is negative, the
     iterator goes backward.
@@ -1170,7 +1019,7 @@ bool QJsonArray::operator!=(const QJsonArray &other) const
     \sa operator-=(), operator+()
 */
 
-/*! \fn QJsonArray::const_iterator &QJsonArray::const_iterator::operator-=(int j)
+/*! \fn QJsonArray::const_iterator &QJsonArray::const_iterator::operator-=(qsizetype j)
 
     Makes the iterator go back by \a j items. If \a j is negative,
     the iterator goes forward.
@@ -1178,7 +1027,7 @@ bool QJsonArray::operator!=(const QJsonArray &other) const
     \sa operator+=(), operator-()
 */
 
-/*! \fn QJsonArray::const_iterator QJsonArray::const_iterator::operator+(int j) const
+/*! \fn QJsonArray::const_iterator QJsonArray::const_iterator::operator+(qsizetype j) const
 
     Returns an iterator to the item at \a j positions forward from
     this iterator. If \a j is negative, the iterator goes backward.
@@ -1186,7 +1035,7 @@ bool QJsonArray::operator!=(const QJsonArray &other) const
     \sa operator-(), operator+=()
 */
 
-/*! \fn QJsonArray::const_iterator QJsonArray::const_iterator::operator-(int j) const
+/*! \fn QJsonArray::const_iterator QJsonArray::const_iterator::operator-(qsizetype j) const
 
     Returns an iterator to the item at \a j positions backward from
     this iterator. If \a j is negative, the iterator goes forward.
@@ -1194,66 +1043,24 @@ bool QJsonArray::operator!=(const QJsonArray &other) const
     \sa operator+(), operator-=()
 */
 
-/*! \fn int QJsonArray::const_iterator::operator-(const_iterator other) const
+/*! \fn qsizetype QJsonArray::const_iterator::operator-(const_iterator other) const
 
     Returns the number of items between the item pointed to by \a
     other and the item pointed to by this iterator.
 */
 
-
 /*!
     \internal
  */
-void QJsonArray::detach(uint reserve)
+bool QJsonArray::detach(qsizetype reserve)
 {
-    Q_UNUSED(reserve)
-    Q_ASSERT(!reserve);
-    detach2(0);
-}
-
-/*!
-    \internal
- */
-bool QJsonArray::detach2(uint reserve)
-{
-    if (!d) {
-        if (reserve >= QJsonPrivate::Value::MaxSize) {
-            qWarning("QJson: Document too large to store in data structure");
-            return false;
-        }
-        d = new QJsonPrivate::Data(reserve, QJsonValue::Array);
-        a = static_cast<QJsonPrivate::Array *>(d->header->root());
-        d->ref.ref();
+    if (!a)
         return true;
-    }
-    if (reserve == 0 && d->ref.loadRelaxed() == 1)
-        return true;
-
-    QJsonPrivate::Data *x = d->clone(a, reserve);
-    if (!x)
-        return false;
-    x->ref.ref();
-    if (!d->ref.deref())
-        delete d;
-    d = x;
-    a = static_cast<QJsonPrivate::Array *>(d->header->root());
-    return true;
+    a = a->detach(a.data(), reserve ? reserve : size());
+    return a;
 }
 
-/*!
-    \internal
- */
-void QJsonArray::compact()
-{
-    if (!d || !d->compactionCounter)
-        return;
-
-    detach2();
-    d->compact();
-    a = static_cast<QJsonPrivate::Array *>(d->header->root());
-}
-
-uint qHash(const QJsonArray &array, uint seed)
+size_t qHash(const QJsonArray &array, size_t seed)
 {
     return qHashRange(array.begin(), array.end(), seed);
 }
@@ -1267,7 +1074,7 @@ QDebug operator<<(QDebug dbg, const QJsonArray &a)
         return dbg;
     }
     QByteArray json;
-    QJsonPrivate::Writer::arrayToJson(a.a, json, 0, true);
+    QJsonPrivate::Writer::arrayToJson(a.a.data(), json, 0, true);
     dbg.nospace() << "QJsonArray("
                   << json.constData() // print as utf-8 string without extra quotation marks
                   << ")";
