@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 //#define QIMAGEREADER_DEBUG
 
@@ -47,7 +11,6 @@
     \inmodule QtGui
     \reentrant
     \ingroup painting
-    \ingroup io
 
     The most common way to read images is through QImage and QPixmap's
     constructors, or by calling QImage::load() and
@@ -101,8 +64,8 @@
     This can be disabled by setting the environment variable
     \c QT_HIGHDPI_DISABLE_2X_IMAGE_LOADING.
 
-    \sa QImageWriter, QImageIOHandler, QImageIOPlugin, QMimeDatabase
-    \sa QImage::devicePixelRatio(), QPixmap::devicePixelRatio(), QIcon, QPainter::drawPixmap(), QPainter::drawImage(), Qt::AA_UseHighDpiPixmaps
+    \sa QImageWriter, QImageIOHandler, QImageIOPlugin, QMimeDatabase, QColorSpace
+    \sa QImage::devicePixelRatio(), QPixmap::devicePixelRatio(), QIcon, QPainter::drawPixmap(), QPainter::drawImage()
 */
 
 /*!
@@ -150,7 +113,7 @@
 // factory loader
 #include <qcoreapplication.h>
 #include <private/qfactoryloader_p.h>
-#include <QMutexLocker>
+#include <QtCore/private/qlocking_p.h>
 
 // for qt_getImageText
 #include <private/qimage_p.h>
@@ -172,6 +135,7 @@
 QT_BEGIN_NAMESPACE
 
 using namespace QImageReaderWriterHelpers;
+using namespace Qt::StringLiterals;
 
 static QImageIOHandler *createReadHandlerHelper(QIODevice *device,
                                                 const QByteArray &format,
@@ -179,15 +143,15 @@ static QImageIOHandler *createReadHandlerHelper(QIODevice *device,
                                                 bool ignoresFormatAndExtension)
 {
     if (!autoDetectImageFormat && format.isEmpty())
-        return 0;
+        return nullptr;
 
     QByteArray form = format.toLower();
-    QImageIOHandler *handler = 0;
+    QImageIOHandler *handler = nullptr;
     QByteArray suffix;
 
 #ifndef QT_NO_IMAGEFORMATPLUGIN
-    static QMutex mutex;
-    QMutexLocker locker(&mutex);
+    Q_CONSTINIT static QBasicMutex mutex;
+    const auto locker = qt_scoped_lock(mutex);
 
     typedef QMultiMap<int, QString> PluginKeyMap;
 
@@ -197,7 +161,7 @@ static QImageIOHandler *createReadHandlerHelper(QIODevice *device,
 
 #ifdef QIMAGEREADER_DEBUG
     qDebug() << "QImageReader::createReadHandler( device =" << (void *)device << ", format =" << format << "),"
-             << keyMap.size() << "plugins available: " << keyMap.values();
+             << keyMap.uniqueKeys().size() << "plugins available: " << keyMap;
 #endif
 
     int suffixPluginIndex = -1;
@@ -271,7 +235,7 @@ static QImageIOHandler *createReadHandlerHelper(QIODevice *device,
                 }
             }
         } else {
-            const int testIndex = keyMap.key(QLatin1String(testFormat), -1);
+            const int testIndex = keyMap.key(QLatin1StringView(testFormat), -1);
             if (testIndex != -1) {
                 QImageIOPlugin *plugin = qobject_cast<QImageIOPlugin *>(l->instance(testIndex));
                 if (plugin && plugin->capabilities(device, testFormat) & QImageIOPlugin::CanRead) {
@@ -325,6 +289,29 @@ static QImageIOHandler *createReadHandlerHelper(QIODevice *device,
 #endif
     }
 
+    if (handler && device && !suffix.isEmpty()) {
+        Q_ASSERT(qobject_cast<QFile *>(device));
+        // We have a file claiming to be of a recognized format. Now confirm that
+        // the handler also recognizes the file contents.
+        const qint64 pos = device->pos();
+        handler->setDevice(device);
+        if (!form.isEmpty())
+            handler->setFormat(form);
+        bool canRead = handler->canRead();
+        device->seek(pos);
+        if (canRead) {
+            // ok, we're done.
+            return handler;
+        }
+#ifdef QIMAGEREADER_DEBUG
+        qDebug() << "QImageReader::createReadHandler: the" << suffix << "handler can not read this file";
+#endif
+        // File may still be valid, just with wrong suffix, so fall back to
+        // finding a handler based on contents, below.
+        delete handler;
+        handler = nullptr;
+    }
+
 #ifndef QT_NO_IMAGEFORMATPLUGIN
     if (!handler && (autoDetectImageFormat || ignoresFormatAndExtension)) {
         // check if any of our plugins recognize the file from its contents.
@@ -336,7 +323,7 @@ static QImageIOHandler *createReadHandlerHelper(QIODevice *device,
                 if (plugin && plugin->capabilities(device, QByteArray()) & QImageIOPlugin::CanRead) {
                     handler = plugin->create(device, testFormat);
 #ifdef QIMAGEREADER_DEBUG
-                    qDebug() << "QImageReader::createReadHandler: the" << keyMap.keys().at(i) << "plugin can read this data";
+                    qDebug() << "QImageReader::createReadHandler: the" << keyMap.value(i) << "plugin can read this data";
 #endif
                     break;
                 }
@@ -427,7 +414,7 @@ static QImageIOHandler *createReadHandlerHelper(QIODevice *device,
         qDebug("QImageReader::createReadHandler: no handlers found. giving up.");
 #endif
         // no handler: give up.
-        return 0;
+        return nullptr;
     }
 
     handler->setDevice(device);
@@ -469,17 +456,21 @@ public:
     QString errorString;
 
     QImageReader *q;
+
+    static int maxAlloc;
 };
 
+int QImageReaderPrivate::maxAlloc = 128; // 128 MB is enough for an 8K 32bpp image
+
 /*!
     \internal
 */
 QImageReaderPrivate::QImageReaderPrivate(QImageReader *qq)
     : autoDetectImageFormat(true), ignoresFormatAndExtension(false)
 {
-    device = 0;
+    device = nullptr;
     deleteDevice = false;
-    handler = 0;
+    handler = nullptr;
     quality = -1;
     imageReaderError = QImageReader::UnknownError;
     autoTransform = UsePluginDefault;
@@ -492,9 +483,9 @@ QImageReaderPrivate::QImageReaderPrivate(QImageReader *qq)
 */
 QImageReaderPrivate::~QImageReaderPrivate()
 {
+    delete handler;
     if (deleteDevice)
         delete device;
-    delete handler;
 }
 
 /*!
@@ -502,6 +493,9 @@ QImageReaderPrivate::~QImageReaderPrivate()
 */
 bool QImageReaderPrivate::initHandler()
 {
+    if (handler)
+        return true;
+
     // check some preconditions
     if (!device || (!deleteDevice && !device->isOpen() && !device->open(QIODevice::ReadOnly))) {
         imageReaderError = QImageReader::DeviceError;
@@ -511,7 +505,7 @@ bool QImageReaderPrivate::initHandler()
 
     // probe the file extension
     if (deleteDevice && !device->isOpen() && !device->open(QIODevice::ReadOnly) && autoDetectImageFormat) {
-        Q_ASSERT(qobject_cast<QFile*>(device) != 0); // future-proofing; for now this should always be the case, so...
+        Q_ASSERT(qobject_cast<QFile*>(device) != nullptr); // future-proofing; for now this should always be the case, so...
         QFile *file = static_cast<QFile *>(device);
 
         if (file->error() == QFileDevice::ResourceError) {
@@ -534,8 +528,8 @@ bool QImageReaderPrivate::initHandler()
         QString fileName = file->fileName();
 
         do {
-            file->setFileName(fileName + QLatin1Char('.')
-                    + QLatin1String(extensions.at(currentExtension++).constData()));
+            file->setFileName(fileName + u'.'
+                    + QLatin1StringView(extensions.at(currentExtension++).constData()));
             file->open(QIODevice::ReadOnly);
         } while (!file->isOpen() && currentExtension < extensions.size());
 
@@ -548,7 +542,7 @@ bool QImageReaderPrivate::initHandler()
     }
 
     // assign a handler
-    if (!handler && (handler = createReadHandlerHelper(device, format, autoDetectImageFormat, ignoresFormatAndExtension)) == 0) {
+    if ((handler = createReadHandlerHelper(device, format, autoDetectImageFormat, ignoresFormatAndExtension)) == nullptr) {
         imageReaderError = QImageReader::UnsupportedFormatError;
         errorString = QImageReader::tr("Unsupported image format");
         return false;
@@ -561,7 +555,7 @@ bool QImageReaderPrivate::initHandler()
 */
 void QImageReaderPrivate::getText()
 {
-    if (text.isEmpty() && (handler || initHandler()) && handler->supportsOption(QImageIOHandler::Description))
+    if (text.isEmpty() && initHandler() && handler->supportsOption(QImageIOHandler::Description))
         text = qt_getImageTextFromDescription(handler->option(QImageIOHandler::Description).toString());
 }
 
@@ -742,7 +736,7 @@ bool QImageReader::decideFormatFromContent() const
     otherwise left unchanged.
 
     If the device is not already open, QImageReader will attempt to
-    open the device in \l QIODevice::ReadOnly mode by calling
+    open the device in \l {QIODeviceBase::}{ReadOnly} mode by calling
     open(). Note that this does not work for certain devices, such as
     QProcess, QTcpSocket and QUdpSocket, where more logic is required
     to open the device.
@@ -751,12 +745,12 @@ bool QImageReader::decideFormatFromContent() const
 */
 void QImageReader::setDevice(QIODevice *device)
 {
+    delete d->handler;
+    d->handler = nullptr;
     if (d->device && d->deleteDevice)
         delete d->device;
     d->device = device;
     d->deleteDevice = false;
-    delete d->handler;
-    d->handler = nullptr;
     d->text.clear();
 }
 
@@ -772,7 +766,7 @@ QIODevice *QImageReader::device() const
 /*!
     Sets the file name of QImageReader to \a fileName. Internally,
     QImageReader will create a QFile object and open it in \l
-    QIODevice::ReadOnly mode, and use this when reading images.
+    {QIODeviceBase::}{ReadOnly} mode, and use this when reading images.
 
     If \a fileName does not include a file extension (e.g., .png or .bmp),
     QImageReader will cycle through all supported extensions until it finds
@@ -1066,7 +1060,7 @@ QList<QByteArray> QImageReader::supportedSubTypes() const
         return QList<QByteArray>();
 
     if (d->handler->supportsOption(QImageIOHandler::SupportedSubTypes))
-        return d->handler->option(QImageIOHandler::SupportedSubTypes).value< QList<QByteArray> >();
+        return qvariant_cast<QList<QByteArray> >(d->handler->option(QImageIOHandler::SupportedSubTypes));
     return QList<QByteArray>();
 }
 
@@ -1115,8 +1109,6 @@ bool QImageReader::autoTransform() const
     case QImageReaderPrivate::DoNotApplyTransform:
         return false;
     case QImageReaderPrivate::UsePluginDefault:
-        if (d->initHandler())
-            return d->handler->supportsOption(QImageIOHandler::TransformedByDefault);
         Q_FALLTHROUGH();
     default:
         break;
@@ -1124,39 +1116,6 @@ bool QImageReader::autoTransform() const
     return false;
 }
 
-/*!
-    \since 5.6
-
-    This is an image format specific function that forces images with
-    gamma information to be gamma corrected to \a gamma. For image formats
-    that do not support gamma correction, this value is ignored.
-
-    To gamma correct to a standard PC color-space, set gamma to \c 1/2.2.
-
-    \sa gamma()
-*/
-void QImageReader::setGamma(float gamma)
-{
-    if (d->initHandler() && d->handler->supportsOption(QImageIOHandler::Gamma))
-        d->handler->setOption(QImageIOHandler::Gamma, gamma);
-}
-
-/*!
-    \since 5.6
-
-    Returns the gamma level of the decoded image. If setGamma() has been
-    called and gamma correction is supported it will return the gamma set.
-    If gamma level is not supported by the image format, \c 0.0 is returned.
-
-    \sa setGamma()
-*/
-float QImageReader::gamma() const
-{
-    if (d->initHandler() && d->handler->supportsOption(QImageIOHandler::Gamma))
-        return d->handler->option(QImageIOHandler::Gamma).toFloat();
-    return 0.0;
-}
-
 /*!
     Returns \c true if an image can be read for the device (i.e., the
     image format is supported, and the device seems to contain valid
@@ -1231,7 +1190,7 @@ bool QImageReader::read(QImage *image)
         return false;
     }
 
-    if (!d->handler && !d->initHandler())
+    if (!d->initHandler())
         return false;
 
     // set the handler specific options.
@@ -1251,11 +1210,9 @@ bool QImageReader::read(QImage *image)
         d->handler->setOption(QImageIOHandler::Quality, d->quality);
 
     // read the image
+    QString filename = fileName();
     if (Q_TRACE_ENABLED(QImageReader_read_before_reading)) {
-        QString fileName = QStringLiteral("unknown");
-        if (QFile *file = qobject_cast<QFile *>(d->device))
-            fileName = file->fileName();
-        Q_TRACE(QImageReader_read_before_reading, this, fileName);
+        Q_TRACE(QImageReader_read_before_reading, this, filename.isEmpty() ? u"unknown"_s : filename);
     }
 
     const bool result = d->handler->read(image);
@@ -1322,8 +1279,8 @@ bool QImageReader::read(QImage *image)
     // successful read; check for "@Nx" file name suffix and set device pixel ratio.
     static bool disableNxImageLoading = !qEnvironmentVariableIsEmpty("QT_HIGHDPI_DISABLE_2X_IMAGE_LOADING");
     if (!disableNxImageLoading) {
-        const QByteArray suffix = QFileInfo(fileName()).baseName().right(3).toLatin1();
-        if (suffix.length() == 3 && suffix[0] == '@' && suffix[1] >= '2' && suffix[1] <= '9' && suffix[2] == 'x')
+        const QByteArray suffix = QFileInfo(filename).baseName().right(3).toLatin1();
+        if (suffix.size() == 3 && suffix[0] == '@' && suffix[1] >= '2' && suffix[1] <= '9' && suffix[2] == 'x')
             image->setDevicePixelRatio(suffix[1] - '0');
     }
     if (autoTransform())
@@ -1583,4 +1540,45 @@ QList<QByteArray> QImageReader::imageFormatsForMimeType(const QByteArray &mimeTy
                                                               QImageReaderWriterHelpers::CanRead);
 }
 
+/*!
+    \since 6.0
+
+    Returns the current allocation limit, in megabytes.
+
+    \sa setAllocationLimit()
+*/
+int QImageReader::allocationLimit()
+{
+    static int envLimit = []() {
+        bool ok = false;
+        int res = qEnvironmentVariableIntValue("QT_IMAGEIO_MAXALLOC", &ok);
+        return ok ? res : -1;
+    }();
+
+    return envLimit >= 0 ? envLimit : QImageReaderPrivate::maxAlloc;
+}
+
+/*!
+    \since 6.0
+
+    Sets the allocation limit to \a mbLimit megabytes. Images that would
+    require a QImage memory allocation above this limit will be rejected.
+    If \a mbLimit is 0, the allocation size check will be disabled.
+
+    This limit helps applications avoid unexpectedly large memory usage from
+    loading corrupt image files. It is normally not needed to change it. The
+    default limit is large enough for all commonly used image sizes.
+
+    \note The memory requirements are calculated for a minimum of 32 bits per pixel, since Qt will
+    typically convert an image to that depth when it is used in GUI. This means that the effective
+    allocation limit is significantly smaller than \a mbLimit when reading 1 bpp and 8 bpp images.
+
+    \sa allocationLimit()
+*/
+void QImageReader::setAllocationLimit(int mbLimit)
+{
+    if (mbLimit >= 0)
+        QImageReaderPrivate::maxAlloc = mbLimit;
+}
+
 QT_END_NAMESPACE
