@@ -9,6 +9,7 @@
 #include <QtCore/qcoreapplication.h>
 #include <QtCore/qfileinfo.h>
 #include <QtCore/qthread.h>
+#include <QtCore/private/qtools_p.h>
 
 #include <private/qnetworkaccessmanager_p.h>
 #include <private/qnetworkfile_p.h>
@@ -68,22 +69,37 @@ QByteArray QNetworkReplyWasmImpl::methodName() const
 
 void QNetworkReplyWasmImpl::close()
 {
+    Q_D(QNetworkReplyWasmImpl);
+
+    if (d->state != QNetworkReplyPrivate::Aborted &&
+        d->state != QNetworkReplyPrivate::Finished &&
+        d->state != QNetworkReplyPrivate::Idle) {
+            d->state = QNetworkReplyPrivate::Finished;
+            d->setCanceled();
+    }
+
     QNetworkReply::close();
-    setFinished(true);
-    emit finished();
 }
 
 void QNetworkReplyWasmImpl::abort()
 {
     Q_D(QNetworkReplyWasmImpl);
+
     if (d->state == QNetworkReplyPrivate::Finished || d->state == QNetworkReplyPrivate::Aborted)
         return;
 
     d->state = QNetworkReplyPrivate::Aborted;
-    d->m_fetch->userData = nullptr;
+    d->setCanceled();
+}
 
-    d->emitReplyError(QNetworkReply::OperationCanceledError, QStringLiteral("Operation canceled"));
-    close();
+void QNetworkReplyWasmImplPrivate::setCanceled()
+{
+    Q_Q(QNetworkReplyWasmImpl);
+    m_fetch->userData = nullptr;
+
+    emitReplyError(QNetworkReply::OperationCanceledError, QStringLiteral("Operation canceled"));
+    q->setFinished(true);
+    emit q->finished();
 }
 
 qint64 QNetworkReplyWasmImpl::bytesAvailable() const
@@ -236,6 +252,7 @@ void QNetworkReplyWasmImplPrivate::doSendRequest()
     attr.destinationPath = destinationPath.constData();
 
     m_fetch = emscripten_fetch(&attr, request.url().toString().toUtf8());
+    state = Working;
 }
 
 void QNetworkReplyWasmImplPrivate::emitReplyError(QNetworkReply::NetworkError errorCode, const QString &errorString)
@@ -284,32 +301,36 @@ static int parseHeaderName(const QByteArray &headerName)
     if (headerName.isEmpty())
         return -1;
 
-    switch (tolower(headerName.at(0))) {
+    auto is = [&](const char *what) {
+        return qstrnicmp(headerName.data(), headerName.size(), what) == 0;
+    };
+
+    switch (QtMiscUtils::toAsciiLower(headerName.front())) {
     case 'c':
-        if (qstricmp(headerName.constData(), "content-type") == 0)
+        if (is("content-type"))
             return QNetworkRequest::ContentTypeHeader;
-        else if (qstricmp(headerName.constData(), "content-length") == 0)
+        else if (is("content-length"))
             return QNetworkRequest::ContentLengthHeader;
-        else if (qstricmp(headerName.constData(), "cookie") == 0)
+        else if (is("cookie"))
             return QNetworkRequest::CookieHeader;
         break;
 
     case 'l':
-        if (qstricmp(headerName.constData(), "location") == 0)
+        if (is("location"))
             return QNetworkRequest::LocationHeader;
-        else if (qstricmp(headerName.constData(), "last-modified") == 0)
+        else if (is("last-modified"))
             return QNetworkRequest::LastModifiedHeader;
         break;
 
     case 's':
-        if (qstricmp(headerName.constData(), "set-cookie") == 0)
+        if (is("set-cookie"))
             return QNetworkRequest::SetCookieHeader;
-        else if (qstricmp(headerName.constData(), "server") == 0)
+        else if (is("server"))
             return QNetworkRequest::ServerHeader;
         break;
 
     case 'u':
-        if (qstricmp(headerName.constData(), "user-agent") == 0)
+        if (is("user-agent"))
             return QNetworkRequest::UserAgentHeader;
         break;
     }
