@@ -1,63 +1,36 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 #include "qibusplatforminputcontext.h"
 
-#include <QtDebug>
+#include <QDebug>
 #include <QTextCharFormat>
 #include <QGuiApplication>
+#include <QWindow>
+#include <QEvent>
+#include <QFile>
+#include <QStandardPaths>
 #include <QDBusVariant>
-#include <qwindow.h>
-#include <qevent.h>
-
-#include <qpa/qplatformcursor.h>
-#include <qpa/qplatformscreen.h>
-#include <qpa/qwindowsysteminterface.h>
+#include <QDBusPendingReply>
+#include <QDBusReply>
+#include <QDBusServiceWatcher>
 
 #include "qibusproxy.h"
 #include "qibusproxyportal.h"
 #include "qibusinputcontextproxy.h"
 #include "qibustypes.h"
 
+#include <qpa/qplatformcursor.h>
+#include <qpa/qplatformscreen.h>
+#include <qpa/qwindowsysteminterface_p.h>
+
+#include <private/qguiapplication_p.h>
+#include <private/qxkbcommon_p.h>
+
+#include <memory>
+
 #include <sys/types.h>
 #include <signal.h>
 
-#include <QtDBus>
 
 #ifndef IBUS_RELEASE_MASK
 #define IBUS_RELEASE_MASK (1 << 30)
@@ -69,30 +42,34 @@
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 enum { debug = 0 };
 
 class QIBusPlatformInputContextPrivate
 {
+    Q_DISABLE_COPY_MOVE(QIBusPlatformInputContextPrivate)
 public:
     QIBusPlatformInputContextPrivate();
     ~QIBusPlatformInputContextPrivate()
     {
-        delete context;
-        delete bus;
-        delete portalBus;
-        delete connection;
+        // dereference QDBusConnection to actually disconnect
+        serviceWatcher.setConnection(QDBusConnection(QString()));
+        context = nullptr;
+        portalBus = nullptr;
+        bus = nullptr;
+        QDBusConnection::disconnectFromBus("QIBusProxy"_L1);
     }
 
     static QString getSocketPath();
 
-    QDBusConnection *createConnection();
+    void createConnection();
     void initBus();
     void createBusProxy();
 
-    QDBusConnection *connection;
-    QIBusProxy *bus;
-    QIBusProxyPortal *portalBus; // bus and portalBus are alternative.
-    QIBusInputContextProxy *context;
+    std::unique_ptr<QIBusProxy> bus;
+    std::unique_ptr<QIBusProxyPortal> portalBus; // bus and portalBus are alternative.
+    std::unique_ptr<QIBusInputContextProxy> context;
     QDBusServiceWatcher serviceWatcher;
 
     bool usePortal; // return value of shouldConnectIbusPortal
@@ -112,7 +89,7 @@ QIBusPlatformInputContext::QIBusPlatformInputContext ()
         QString socketPath = QIBusPlatformInputContextPrivate::getSocketPath();
         QFile file(socketPath);
         if (file.open(QFile::ReadOnly)) {
-#ifndef QT_NO_FILESYSTEMWATCHER
+#if QT_CONFIG(filesystemwatcher)
             qCDebug(qtQpaInputMethods) << "socketWatcher.addPath" << socketPath;
             // If KDE session save is used or restart ibus-daemon,
             // the applications could run before ibus-daemon runs.
@@ -173,8 +150,6 @@ void QIBusPlatformInputContext::invokeAction(QInputMethod::Action a, int)
 
 void QIBusPlatformInputContext::reset()
 {
-    QPlatformInputContext::reset();
-
     if (!d->busConnected)
         return;
 
@@ -185,8 +160,6 @@ void QIBusPlatformInputContext::reset()
 
 void QIBusPlatformInputContext::commit()
 {
-    QPlatformInputContext::commit();
-
     if (!d->busConnected)
         return;
 
@@ -217,17 +190,14 @@ void QIBusPlatformInputContext::update(Qt::InputMethodQueries q)
             && (q.testFlag(Qt::ImSurroundingText)
                 || q.testFlag(Qt::ImCursorPosition)
                 || q.testFlag(Qt::ImAnchorPosition))) {
-        QInputMethodQueryEvent srrndTextQuery(Qt::ImSurroundingText);
-        QInputMethodQueryEvent cursorPosQuery(Qt::ImCursorPosition);
-        QInputMethodQueryEvent anchorPosQuery(Qt::ImAnchorPosition);
 
-        QCoreApplication::sendEvent(input, &srrndTextQuery);
-        QCoreApplication::sendEvent(input, &cursorPosQuery);
-        QCoreApplication::sendEvent(input, &anchorPosQuery);
+        QInputMethodQueryEvent query(Qt::ImSurroundingText | Qt::ImCursorPosition | Qt::ImAnchorPosition);
 
-        QString surroundingText = srrndTextQuery.value(Qt::ImSurroundingText).toString();
-        uint cursorPosition = cursorPosQuery.value(Qt::ImCursorPosition).toUInt();
-        uint anchorPosition = anchorPosQuery.value(Qt::ImAnchorPosition).toUInt();
+        QCoreApplication::sendEvent(input, &query);
+
+        QString surroundingText = query.value(Qt::ImSurroundingText).toString();
+        uint cursorPosition = query.value(Qt::ImCursorPosition).toUInt();
+        uint anchorPosition = query.value(Qt::ImAnchorPosition).toUInt();
 
         QIBusText text;
         text.text = surroundingText;
@@ -238,7 +208,6 @@ void QIBusPlatformInputContext::update(Qt::InputMethodQueries q)
 
         d->context->SetSurroundingText(dbusText, cursorPosition, anchorPosition);
     }
-    QPlatformInputContext::update(q);
 }
 
 void QIBusPlatformInputContext::cursorRectChanged()
@@ -247,16 +216,37 @@ void QIBusPlatformInputContext::cursorRectChanged()
         return;
 
     QRect r = qApp->inputMethod()->cursorRectangle().toRect();
-    if(!r.isValid())
+    if (!r.isValid())
         return;
 
     QWindow *inputWindow = qApp->focusWindow();
     if (!inputWindow)
         return;
-    r.moveTopLeft(inputWindow->mapToGlobal(r.topLeft()));
+    if (!inputWindow->screen())
+        return;
+
+    if (QGuiApplication::platformName().startsWith("wayland"_L1)) {
+        auto margins = inputWindow->frameMargins();
+        r.translate(margins.left(), margins.top());
+        qreal scale = inputWindow->devicePixelRatio();
+        QRect newRect = QRect(r.x() * scale, r.y() * scale, r.width() * scale, r.height() * scale);
+        if (debug)
+            qDebug() << "microFocus" << newRect;
+        d->context->SetCursorLocationRelative(newRect.x(), newRect.y(),
+                                              newRect.width(), newRect.height());
+        return;
+    }
+
+    // x11/xcb
+    auto screenGeometry = inputWindow->screen()->geometry();
+    auto point = inputWindow->mapToGlobal(r.topLeft());
+    qreal scale = inputWindow->devicePixelRatio();
+    auto native = (point - screenGeometry.topLeft()) * scale + screenGeometry.topLeft();
+    QRect newRect(native, r.size() * scale);
     if (debug)
-        qDebug() << "microFocus" << r;
-    d->context->SetCursorLocation(r.x(), r.y(), r.width(), r.height());
+        qDebug() << "microFocus" << newRect;
+    d->context->SetCursorLocation(newRect.x(), newRect.y(),
+                                  newRect.width(), newRect.height());
 }
 
 void QIBusPlatformInputContext::setFocusObject(QObject *object)
@@ -267,7 +257,7 @@ void QIBusPlatformInputContext::setFocusObject(QObject *object)
     // It would seem natural here to call FocusOut() on the input method if we
     // transition from an IME accepted focus object to one that does not accept it.
     // Mysteriously however that is not sufficient to fix bug QTBUG-63066.
-    if (!inputMethodAccepted())
+    if (object && !inputMethodAccepted())
         return;
 
     if (debug)
@@ -284,7 +274,7 @@ void QIBusPlatformInputContext::commitText(const QDBusVariant &text)
     if (!input)
         return;
 
-    const QDBusArgument arg = text.variant().value<QDBusArgument>();
+    const QDBusArgument arg = qvariant_cast<QDBusArgument>(text.variant());
 
     QIBusText t;
     if (debug)
@@ -310,7 +300,7 @@ void QIBusPlatformInputContext::updatePreeditText(const QDBusVariant &text, uint
     if (!input)
         return;
 
-    const QDBusArgument arg = text.variant().value<QDBusArgument>();
+    const QDBusArgument arg = qvariant_cast<QDBusArgument>(text.variant());
 
     QIBusText t;
     arg >> t;
@@ -336,14 +326,12 @@ void QIBusPlatformInputContext::forwardKeyEvent(uint keyval, uint keycode, uint
     if (!input)
         return;
 
-    if (debug)
-        qDebug() << "forwardKeyEvent" << keyval << keycode << state;
-
     QEvent::Type type = QEvent::KeyPress;
     if (state & IBUS_RELEASE_MASK)
         type = QEvent::KeyRelease;
 
     state &= ~IBUS_RELEASE_MASK;
+    keycode += 8;
 
     Qt::KeyboardModifiers modifiers = Qt::NoModifier;
     if (state & IBUS_SHIFT_MASK)
@@ -355,7 +343,13 @@ void QIBusPlatformInputContext::forwardKeyEvent(uint keyval, uint keycode, uint
     if (state & IBUS_META_MASK)
         modifiers |= Qt::MetaModifier;
 
-    QKeyEvent event(type, keyval, modifiers, QString(keyval));
+    int qtcode = QXkbCommon::keysymToQtKey(keyval, modifiers);
+    QString text = QXkbCommon::lookupStringNoKeysymTransformations(keyval);
+
+    if (debug)
+        qDebug() << "forwardKeyEvent" << keyval << keycode << state << modifiers << qtcode << text;
+
+    QKeyEvent event(type, qtcode, modifiers, keycode, keyval, state, text);
     QCoreApplication::sendEvent(input, &event);
 }
 
@@ -422,9 +416,9 @@ bool QIBusPlatformInputContext::filterEvent(const QEvent *event)
     QDBusPendingReply<bool> reply = d->context->ProcessKeyEvent(sym, code - 8, ibusState);
 
     if (m_eventFilterUseSynchronousMode || reply.isFinished()) {
-        bool retval = reply.value();
-        qCDebug(qtQpaInputMethods) << "filterEvent return" << code << sym << state << retval;
-        return retval;
+        bool filtered = reply.value();
+        qCDebug(qtQpaInputMethods) << "filterEvent return" << code << sym << state << filtered;
+        return filtered;
     }
 
     Qt::KeyboardModifiers modifiers = keyEvent->modifiers();
@@ -494,23 +488,22 @@ void QIBusPlatformInputContext::filterEventFinished(QDBusPendingCallWatcher *cal
     const bool isAutoRepeat = args.at(7).toBool();
 
     // copied from QXcbKeyboard::handleKeyEvent()
-    bool retval = reply.value();
-    qCDebug(qtQpaInputMethods) << "filterEventFinished return" << code << sym << state << retval;
-    if (!retval) {
+    bool filtered = reply.value();
+    qCDebug(qtQpaInputMethods) << "filterEventFinished return" << code << sym << state << filtered;
+    if (!filtered) {
 #ifndef QT_NO_CONTEXTMENU
         if (type == QEvent::KeyPress && qtcode == Qt::Key_Menu
             && window != NULL) {
             const QPoint globalPos = window->screen()->handle()->cursor()->pos();
             const QPoint pos = window->mapFromGlobal(globalPos);
-#ifndef QT_NO_CONTEXTMENU
-            QWindowSystemInterface::handleContextMenuEvent(window, false, pos,
-                                                           globalPos, modifiers);
-#endif
+            QWindowSystemInterfacePrivate::ContextMenuEvent contextMenuEvent(window, false, pos,
+                                                                             globalPos, modifiers);
+            QGuiApplicationPrivate::processWindowSystemEvent(&contextMenuEvent);
         }
-#endif // QT_NO_CONTEXTMENU
-        QWindowSystemInterface::handleExtendedKeyEvent(window, time, type, qtcode, modifiers,
-                                                       code, sym, state, string, isAutoRepeat);
-
+#endif
+        QWindowSystemInterfacePrivate::KeyEvent keyEvent(window, time, type, qtcode, modifiers,
+                                                         code, sym, state, string, isAutoRepeat);
+        QGuiApplicationPrivate::processWindowSystemEvent(&keyEvent);
     }
     call->deleteLater();
 }
@@ -530,12 +523,12 @@ void QIBusPlatformInputContext::socketChanged(const QString &str)
 
     m_timer.stop();
 
-    if (d->context)
-        disconnect(d->context);
-    if (d->bus && d->bus->isValid())
-        disconnect(d->bus);
-    if (d->connection)
-        d->connection->disconnectFromBus(QLatin1String("QIBusProxy"));
+    // dereference QDBusConnection to actually disconnect
+    d->serviceWatcher.setConnection(QDBusConnection(QString()));
+    d->context = nullptr;
+    d->bus = nullptr;
+    d->busConnected = false;
+    QDBusConnection::disconnectFromBus("QIBusProxy"_L1);
 
     m_timer.start(100);
 }
@@ -564,7 +557,7 @@ void QIBusPlatformInputContext::connectToBus()
     d->initBus();
     connectToContextSignals();
 
-#ifndef QT_NO_FILESYSTEMWATCHER
+#if QT_CONFIG(filesystemwatcher)
     if (!d->usePortal && m_socketWatcher.files().size() == 0)
         m_socketWatcher.addPath(QIBusPlatformInputContextPrivate::getSocketPath());
 #endif
@@ -587,23 +580,23 @@ void QIBusPlatformInputContext::globalEngineChanged(const QString &engine_name)
 void QIBusPlatformInputContext::connectToContextSignals()
 {
     if (d->bus && d->bus->isValid()) {
-        connect(d->bus, SIGNAL(GlobalEngineChanged(QString)), this, SLOT(globalEngineChanged(QString)));
+        connect(d->bus.get(), SIGNAL(GlobalEngineChanged(QString)), this, SLOT(globalEngineChanged(QString)));
     }
 
     if (d->context) {
-        connect(d->context, SIGNAL(CommitText(QDBusVariant)), SLOT(commitText(QDBusVariant)));
-        connect(d->context, SIGNAL(UpdatePreeditText(QDBusVariant,uint,bool)), this, SLOT(updatePreeditText(QDBusVariant,uint,bool)));
-        connect(d->context, SIGNAL(ForwardKeyEvent(uint, uint, uint)), this, SLOT(forwardKeyEvent(uint, uint, uint)));
-        connect(d->context, SIGNAL(DeleteSurroundingText(int,uint)), this, SLOT(deleteSurroundingText(int,uint)));
-        connect(d->context, SIGNAL(RequireSurroundingText()), this, SLOT(surroundingTextRequired()));
-        connect(d->context, SIGNAL(HidePreeditText()), this, SLOT(hidePreeditText()));
-        connect(d->context, SIGNAL(ShowPreeditText()), this, SLOT(showPreeditText()));
+        connect(d->context.get(), SIGNAL(CommitText(QDBusVariant)), SLOT(commitText(QDBusVariant)));
+        connect(d->context.get(), SIGNAL(UpdatePreeditText(QDBusVariant,uint,bool)), this, SLOT(updatePreeditText(QDBusVariant,uint,bool)));
+        connect(d->context.get(), SIGNAL(ForwardKeyEvent(uint,uint,uint)), this, SLOT(forwardKeyEvent(uint,uint,uint)));
+        connect(d->context.get(), SIGNAL(DeleteSurroundingText(int,uint)), this, SLOT(deleteSurroundingText(int,uint)));
+        connect(d->context.get(), SIGNAL(RequireSurroundingText()), this, SLOT(surroundingTextRequired()));
+        connect(d->context.get(), SIGNAL(HidePreeditText()), this, SLOT(hidePreeditText()));
+        connect(d->context.get(), SIGNAL(ShowPreeditText()), this, SLOT(showPreeditText()));
     }
 }
 
 static inline bool checkRunningUnderFlatpak()
 {
-    return !QStandardPaths::locate(QStandardPaths::RuntimeLocation, QLatin1String("flatpak-info")).isEmpty();
+    return !QStandardPaths::locate(QStandardPaths::RuntimeLocation, "flatpak-info"_L1).isEmpty();
 }
 
 static bool shouldConnectIbusPortal()
@@ -613,11 +606,7 @@ static bool shouldConnectIbusPortal()
 }
 
 QIBusPlatformInputContextPrivate::QIBusPlatformInputContextPrivate()
-    : connection(0),
-      bus(0),
-      portalBus(0),
-      context(0),
-      usePortal(shouldConnectIbusPortal()),
+    : usePortal(shouldConnectIbusPortal()),
       valid(false),
       busConnected(false),
       needsSurroundingText(false)
@@ -641,42 +630,43 @@ QIBusPlatformInputContextPrivate::QIBusPlatformInputContextPrivate()
 
 void QIBusPlatformInputContextPrivate::initBus()
 {
-    connection = createConnection();
+    createConnection();
     busConnected = false;
     createBusProxy();
 }
 
 void QIBusPlatformInputContextPrivate::createBusProxy()
 {
-    if (!connection || !connection->isConnected())
+    QDBusConnection connection("QIBusProxy"_L1);
+    if (!connection.isConnected())
         return;
 
     const char* ibusService = usePortal ? "org.freedesktop.portal.IBus" : "org.freedesktop.IBus";
     QDBusReply<QDBusObjectPath> ic;
     if (usePortal) {
-        portalBus = new QIBusProxyPortal(QLatin1String(ibusService),
-                                         QLatin1String("/org/freedesktop/IBus"),
-                                         *connection);
+        portalBus = std::make_unique<QIBusProxyPortal>(QLatin1StringView(ibusService),
+                                                       "/org/freedesktop/IBus"_L1,
+                                                       connection);
         if (!portalBus->isValid()) {
             qWarning("QIBusPlatformInputContext: invalid portal bus.");
             return;
         }
 
-        ic = portalBus->CreateInputContext(QLatin1String("QIBusInputContext"));
+        ic = portalBus->CreateInputContext("QIBusInputContext"_L1);
     } else {
-        bus = new QIBusProxy(QLatin1String(ibusService),
-                             QLatin1String("/org/freedesktop/IBus"),
-                             *connection);
+        bus = std::make_unique<QIBusProxy>(QLatin1StringView(ibusService),
+                                           "/org/freedesktop/IBus"_L1,
+                                           connection);
         if (!bus->isValid()) {
             qWarning("QIBusPlatformInputContext: invalid bus.");
             return;
         }
 
-        ic = bus->CreateInputContext(QLatin1String("QIBusInputContext"));
+        ic = bus->CreateInputContext("QIBusInputContext"_L1);
     }
 
     serviceWatcher.removeWatchedService(ibusService);
-    serviceWatcher.setConnection(*connection);
+    serviceWatcher.setConnection(connection);
     serviceWatcher.addWatchedService(ibusService);
 
     if (!ic.isValid()) {
@@ -684,7 +674,7 @@ void QIBusPlatformInputContextPrivate::createBusProxy()
         return;
     }
 
-    context = new QIBusInputContextProxy(QLatin1String(ibusService), ic.value().path(), *connection);
+    context = std::make_unique<QIBusInputContextProxy>(QLatin1StringView(ibusService), ic.value().path(), connection);
 
     if (!context->isValid()) {
         qWarning("QIBusPlatformInputContext: invalid input context.");
@@ -708,36 +698,54 @@ void QIBusPlatformInputContextPrivate::createBusProxy()
 
 QString QIBusPlatformInputContextPrivate::getSocketPath()
 {
-    QByteArray display(qgetenv("DISPLAY"));
-    QByteArray host = "unix";
+    QByteArray display;
     QByteArray displayNumber = "0";
+    bool isWayland = false;
+
+    if (qEnvironmentVariableIsSet("IBUS_ADDRESS_FILE")) {
+        QByteArray path = qgetenv("IBUS_ADDRESS_FILE");
+        return QString::fromLocal8Bit(path);
+    } else  if (qEnvironmentVariableIsSet("WAYLAND_DISPLAY")) {
+        display = qgetenv("WAYLAND_DISPLAY");
+        isWayland = true;
+    } else {
+        display = qgetenv("DISPLAY");
+    }
+    QByteArray host = "unix";
+
+    if (isWayland) {
+        displayNumber = display;
+    } else {
+        int pos = display.indexOf(':');
+        if (pos > 0)
+            host = display.left(pos);
+        ++pos;
+        int pos2 = display.indexOf('.', pos);
+        if (pos2 > 0)
+            displayNumber = display.mid(pos, pos2 - pos);
+         else
+            displayNumber = display.mid(pos);
+    }
 
-    int pos = display.indexOf(':');
-    if (pos > 0)
-        host = display.left(pos);
-    ++pos;
-    int pos2 = display.indexOf('.', pos);
-    if (pos2 > 0)
-        displayNumber = display.mid(pos, pos2 - pos);
-    else
-        displayNumber = display.mid(pos);
     if (debug)
         qDebug() << "host=" << host << "displayNumber" << displayNumber;
 
     return QStandardPaths::writableLocation(QStandardPaths::ConfigLocation) +
-               QLatin1String("/ibus/bus/") +
-               QLatin1String(QDBusConnection::localMachineId()) +
-               QLatin1Char('-') + QString::fromLocal8Bit(host) + QLatin1Char('-') + QString::fromLocal8Bit(displayNumber);
+               "/ibus/bus/"_L1 +
+               QLatin1StringView(QDBusConnection::localMachineId()) +
+               u'-' + QString::fromLocal8Bit(host) + u'-' + QString::fromLocal8Bit(displayNumber);
 }
 
-QDBusConnection *QIBusPlatformInputContextPrivate::createConnection()
+void QIBusPlatformInputContextPrivate::createConnection()
 {
-    if (usePortal)
-        return new QDBusConnection(QDBusConnection::connectToBus(QDBusConnection::SessionBus, QLatin1String("QIBusProxy")));
-    QFile file(getSocketPath());
+    if (usePortal) {
+        QDBusConnection::connectToBus(QDBusConnection::SessionBus, "QIBusProxy"_L1);
+        return;
+    }
 
+    QFile file(getSocketPath());
     if (!file.open(QFile::ReadOnly))
-        return 0;
+        return;
 
     QByteArray address;
     int pid = -1;
@@ -756,9 +764,11 @@ QDBusConnection *QIBusPlatformInputContextPrivate::createConnection()
     if (debug)
         qDebug() << "IBUS_ADDRESS=" << address << "PID=" << pid;
     if (address.isEmpty() || pid < 0 || kill(pid, 0) != 0)
-        return 0;
+        return;
 
-    return new QDBusConnection(QDBusConnection::connectToBus(QString::fromLatin1(address), QLatin1String("QIBusProxy")));
+    QDBusConnection::connectToBus(QString::fromLatin1(address), "QIBusProxy"_L1);
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qibusplatforminputcontext.cpp"
