@@ -1,46 +1,11 @@
-/****************************************************************************
-**
-** Copyright (C) 2020 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtSql module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2020 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qsql_mysql_p.h"
 
 #include <qcoreapplication.h>
 #include <qvariant.h>
+#include <qvarlengtharray.h>
 #include <qdatetime.h>
 #include <qdebug.h>
 #include <qfile.h>
@@ -64,12 +29,30 @@ Q_DECLARE_METATYPE(MYSQL_RES*)
 Q_DECLARE_METATYPE(MYSQL*)
 Q_DECLARE_METATYPE(MYSQL_STMT*)
 
+// MYSQL_TYPE_JSON was introduced with MySQL 5.7.9
+#if defined(MYSQL_VERSION_ID) && MYSQL_VERSION_ID < 50709
+#define MYSQL_TYPE_JSON  245
+#endif
+
 // MySQL above version 8 removed my_bool typedef while MariaDB kept it,
 // by redefining it we can regain source compatibility.
 using my_bool = decltype(mysql_stmt_bind_result(nullptr, nullptr));
 
+// this is a copy of the old MYSQL_TIME before an additional integer was added in
+// 8.0.27.0. This kills the sanity check during retrieving this struct from mysql
+// when another libmysql version is used during runtime than during compile time
+struct QT_MYSQL_TIME
+{
+    unsigned int year, month, day, hour, minute, second;
+    unsigned long second_part; /**< microseconds */
+    my_bool neg;
+    enum enum_mysql_timestamp_type time_type;
+};
+
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 class QMYSQLDriverPrivate : public QSqlDriverPrivate
 {
     Q_DECLARE_PUBLIC(QMYSQLDriver)
@@ -113,10 +96,9 @@ static inline QVariant qDateTimeFromString(QString &val)
 #else
     if (val.isEmpty())
         return QVariant(QDateTime());
-    if (val.length() == 14)
+    if (val.size() == 14)
         // TIMESTAMPS have the format yyyyMMddhhmmss
-        val.insert(4, QLatin1Char('-')).insert(7, QLatin1Char('-')).insert(10,
-                    QLatin1Char('T')).insert(13, QLatin1Char(':')).insert(16, QLatin1Char(':'));
+        val.insert(4, u'-').insert(7, u'-').insert(10, u'T').insert(13, u':').insert(16, u':');
     return QVariant(QDateTime::fromString(val, Qt::ISODate));
 #endif
 }
@@ -209,64 +191,65 @@ static QSqlError qMakeError(const QString& err, QSqlError::ErrorType type,
                             const QMYSQLDriverPrivate* p)
 {
     const char *cerr = p->mysql ? mysql_error(p->mysql) : 0;
-    return QSqlError(QLatin1String("QMYSQL: ") + err,
+    return QSqlError("QMYSQL: "_L1 + err,
                      QString::fromUtf8(cerr),
                      type, QString::number(mysql_errno(p->mysql)));
 }
 
 
-static QMetaType qDecodeMYSQLType(int mysqltype, uint flags)
+static QMetaType qDecodeMYSQLType(enum_field_types mysqltype, uint flags)
 {
     QMetaType::Type type;
     switch (mysqltype) {
-    case FIELD_TYPE_TINY :
+    case MYSQL_TYPE_TINY:
         type = (flags & UNSIGNED_FLAG) ? QMetaType::UChar : QMetaType::Char;
         break;
-    case FIELD_TYPE_SHORT :
+    case MYSQL_TYPE_SHORT:
         type = (flags & UNSIGNED_FLAG) ? QMetaType::UShort : QMetaType::Short;
         break;
-    case FIELD_TYPE_LONG :
-    case FIELD_TYPE_INT24 :
+    case MYSQL_TYPE_LONG:
+    case MYSQL_TYPE_INT24:
         type = (flags & UNSIGNED_FLAG) ? QMetaType::UInt : QMetaType::Int;
         break;
-    case FIELD_TYPE_YEAR :
+    case MYSQL_TYPE_YEAR:
         type = QMetaType::Int;
         break;
-    case FIELD_TYPE_LONGLONG :
+    case MYSQL_TYPE_LONGLONG:
         type = (flags & UNSIGNED_FLAG) ? QMetaType::ULongLong : QMetaType::LongLong;
         break;
-    case FIELD_TYPE_FLOAT :
-    case FIELD_TYPE_DOUBLE :
-    case FIELD_TYPE_DECIMAL :
-#if defined(FIELD_TYPE_NEWDECIMAL)
-    case FIELD_TYPE_NEWDECIMAL:
-#endif
+    case MYSQL_TYPE_FLOAT:
+    case MYSQL_TYPE_DOUBLE:
+    case MYSQL_TYPE_DECIMAL:
+    case MYSQL_TYPE_NEWDECIMAL:
         type = QMetaType::Double;
         break;
-    case FIELD_TYPE_DATE :
+    case MYSQL_TYPE_DATE:
         type = QMetaType::QDate;
         break;
-    case FIELD_TYPE_TIME :
+    case MYSQL_TYPE_TIME:
         // A time field can be within the range '-838:59:59' to '838:59:59' so
         // use QString instead of QTime since QTime is limited to 24 hour clock
         type = QMetaType::QString;
         break;
-    case FIELD_TYPE_DATETIME :
-    case FIELD_TYPE_TIMESTAMP :
+    case MYSQL_TYPE_DATETIME:
+    case MYSQL_TYPE_TIMESTAMP:
         type = QMetaType::QDateTime;
         break;
-    case FIELD_TYPE_STRING :
-    case FIELD_TYPE_VAR_STRING :
-    case FIELD_TYPE_BLOB :
-    case FIELD_TYPE_TINY_BLOB :
-    case FIELD_TYPE_MEDIUM_BLOB :
-    case FIELD_TYPE_LONG_BLOB :
-    case FIELD_TYPE_GEOMETRY :
+    case MYSQL_TYPE_STRING:
+    case MYSQL_TYPE_VAR_STRING:
+    case MYSQL_TYPE_BLOB:
+    case MYSQL_TYPE_TINY_BLOB:
+    case MYSQL_TYPE_MEDIUM_BLOB:
+    case MYSQL_TYPE_LONG_BLOB:
+    case MYSQL_TYPE_GEOMETRY:
+    case MYSQL_TYPE_JSON:
         type = (flags & BINARY_FLAG) ? QMetaType::QByteArray : QMetaType::QString;
         break;
-    default:
-    case FIELD_TYPE_ENUM :
-    case FIELD_TYPE_SET :
+    case MYSQL_TYPE_ENUM:
+    case MYSQL_TYPE_SET:
+        type = QMetaType::QString;
+        break;
+    default:  // needed because there are more enum values which are not available in all headers
         type = QMetaType::QString;
         break;
     }
@@ -276,7 +259,7 @@ static QMetaType qDecodeMYSQLType(int mysqltype, uint flags)
 static QSqlField qToField(MYSQL_FIELD *field)
 {
     QSqlField f(QString::fromUtf8(field->name),
-                qDecodeMYSQLType(int(field->type), field->flags),
+                qDecodeMYSQLType(field->type, field->flags),
                 QString::fromUtf8(field->table));
     f.setRequired(IS_NOT_NULL(field->flags));
     f.setLength(field->length);
@@ -290,20 +273,21 @@ static QSqlError qMakeStmtError(const QString& err, QSqlError::ErrorType type,
                             MYSQL_STMT* stmt)
 {
     const char *cerr = mysql_stmt_error(stmt);
-    return QSqlError(QLatin1String("QMYSQL: ") + err,
+    return QSqlError("QMYSQL: "_L1 + err,
                      QString::fromLatin1(cerr),
                      type, QString::number(mysql_stmt_errno(stmt)));
 }
 
-static bool qIsBlob(int t)
+static bool qIsBlob(enum_field_types t)
 {
     return t == MYSQL_TYPE_TINY_BLOB
            || t == MYSQL_TYPE_BLOB
            || t == MYSQL_TYPE_MEDIUM_BLOB
-           || t == MYSQL_TYPE_LONG_BLOB;
+           || t == MYSQL_TYPE_LONG_BLOB
+           || t == MYSQL_TYPE_JSON;
 }
 
-static bool qIsTimeOrDate(int t)
+static bool qIsTimeOrDate(enum_field_types t)
 {
     // *not* MYSQL_TYPE_TIME because its range is bigger than QTime
     // (see above)
@@ -320,7 +304,7 @@ static bool qIsInteger(int t)
 
 void QMYSQLResultPrivate::bindBlobs()
 {
-    for (int i = 0; i < fields.count(); ++i) {
+    for (int i = 0; i < fields.size(); ++i) {
         const MYSQL_FIELD *fieldInfo = fields.at(i).myField;
         if (qIsBlob(inBinds[i].buffer_type) && meta && fieldInfo) {
             MYSQL_BIND *bind = &inBinds[i];
@@ -362,7 +346,7 @@ bool QMYSQLResultPrivate::bindInValues()
             bind->buffer_length = f.bufLength = 0;
             hasBlobs = true;
         } else if (qIsTimeOrDate(fieldInfo->type)) {
-            bind->buffer_length = f.bufLength = sizeof(MYSQL_TIME);
+            bind->buffer_length = f.bufLength = sizeof(QT_MYSQL_TIME);
         } else if (qIsInteger(f.type.id())) {
             bind->buffer_length = f.bufLength = 8;
         } else {
@@ -406,7 +390,7 @@ void QMYSQLResult::cleanup()
     if (d->result)
         mysql_free_result(d->result);
 
-// must iterate trough leftover result sets from multi-selects or stored procedures
+// must iterate through leftover result sets from multi-selects or stored procedures
 // if this isn't done subsequent queries will fail with "Commands out of sync"
     while (driver() && d->drv_d_func()->mysql && mysql_next_result(d->drv_d_func()->mysql) == 0) {
         MYSQL_RES *res = mysql_store_result(d->drv_d_func()->mysql);
@@ -426,7 +410,7 @@ void QMYSQLResult::cleanup()
     }
 
     int i;
-    for (i = 0; i < d->fields.count(); ++i)
+    for (i = 0; i < d->fields.size(); ++i)
         delete[] d->fields[i].outField;
 
     if (d->outBinds) {
@@ -543,7 +527,7 @@ bool QMYSQLResult::fetchFirst()
 QVariant QMYSQLResult::data(int field)
 {
     Q_D(QMYSQLResult);
-    if (!isSelect() || field >= d->fields.count()) {
+    if (!isSelect() || field >= d->fields.size()) {
         qWarning("QMYSQLResult::data: column %d out of range", field);
         return QVariant();
     }
@@ -566,8 +550,8 @@ QVariant QMYSQLResult::data(int field)
             else if (f.type.id() == QMetaType::Char)
                 return variant.toInt();
             return variant;
-        } else if (qIsTimeOrDate(f.myField->type) && f.bufLength == sizeof(MYSQL_TIME)) {
-            auto t = reinterpret_cast<const MYSQL_TIME *>(f.outField);
+        } else if (qIsTimeOrDate(f.myField->type) && f.bufLength >= sizeof(QT_MYSQL_TIME)) {
+            auto t = reinterpret_cast<const QT_MYSQL_TIME *>(f.outField);
             QDate date;
             QTime time;
             if (f.type.id() != QMetaType::QTime)
@@ -659,7 +643,7 @@ QVariant QMYSQLResult::data(int field)
 bool QMYSQLResult::isNull(int field)
 {
    Q_D(const QMYSQLResult);
-   if (field < 0 || field >= d->fields.count())
+   if (field < 0 || field >= d->fields.size())
        return true;
    if (d->preparedQuery)
        return d->fields.at(field).nullIndicator;
@@ -678,7 +662,7 @@ bool QMYSQLResult::reset (const QString& query)
     cleanup();
 
     const QByteArray encQuery = query.toUtf8();
-    if (mysql_real_query(d->drv_d_func()->mysql, encQuery.data(), encQuery.length())) {
+    if (mysql_real_query(d->drv_d_func()->mysql, encQuery.data(), encQuery.size())) {
         setLastError(qMakeError(QCoreApplication::translate("QMYSQLResult", "Unable to execute query"),
                      QSqlError::StatementError, d->drv_d_func()));
         return false;
@@ -786,7 +770,7 @@ bool QMYSQLResult::nextResult()
     d->result = 0;
     setSelect(false);
 
-    for (int i = 0; i < d->fields.count(); ++i)
+    for (int i = 0; i < d->fields.size(); ++i)
         delete[] d->fields[i].outField;
     d->fields.clear();
 
@@ -827,13 +811,12 @@ void QMYSQLResult::virtual_hook(int id, void *data)
     QSqlResult::virtual_hook(id, data);
 }
 
-static MYSQL_TIME *toMySqlDate(QDate date, QTime time, int type)
+static QT_MYSQL_TIME *toMySqlDate(QDate date, QTime time, int type)
 {
     Q_ASSERT(type == QMetaType::QTime || type == QMetaType::QDate
              || type == QMetaType::QDateTime);
 
-    MYSQL_TIME *myTime = new MYSQL_TIME;
-    memset(myTime, 0, sizeof(MYSQL_TIME));
+    auto myTime = new QT_MYSQL_TIME{};
 
     if (type == QMetaType::QTime || type == QMetaType::QDateTime) {
         myTime->hour = time.hour();
@@ -874,7 +857,7 @@ bool QMYSQLResult::prepare(const QString& query)
     }
 
     const QByteArray encQuery = query.toUtf8();
-    r = mysql_stmt_prepare(d->stmt, encQuery.constData(), encQuery.length());
+    r = mysql_stmt_prepare(d->stmt, encQuery.constData(), encQuery.size());
     if (r != 0) {
         setLastError(qMakeStmtError(QCoreApplication::translate("QMYSQLResult",
                      "Unable to prepare statement"), QSqlError::StatementError, d->stmt));
@@ -902,7 +885,7 @@ bool QMYSQLResult::exec()
         return false;
 
     int r = 0;
-    QList<MYSQL_TIME *> timeVector;
+    QList<QT_MYSQL_TIME *> timeVector;
     QList<QByteArray> stringVector;
     QList<my_bool> nullVector;
 
@@ -916,10 +899,10 @@ bool QMYSQLResult::exec()
     }
 
     if (mysql_stmt_param_count(d->stmt) > 0 &&
-        mysql_stmt_param_count(d->stmt) == (uint)values.count()) {
+        mysql_stmt_param_count(d->stmt) == (uint)values.size()) {
 
-        nullVector.resize(values.count());
-        for (int i = 0; i < values.count(); ++i) {
+        nullVector.resize(values.size());
+        for (int i = 0; i < values.size(); ++i) {
             const QVariant &val = boundValues().at(i);
             void *data = const_cast<void *>(val.constData());
 
@@ -940,7 +923,7 @@ bool QMYSQLResult::exec()
                 case QMetaType::QTime:
                 case QMetaType::QDate:
                 case QMetaType::QDateTime: {
-                    MYSQL_TIME *myTime = toMySqlDate(val.toDate(), val.toTime(), val.userType());
+                    QT_MYSQL_TIME *myTime = toMySqlDate(val.toDate(), val.toTime(), val.userType());
                     timeVector.append(myTime);
 
                     currBind->buffer = myTime;
@@ -960,7 +943,7 @@ bool QMYSQLResult::exec()
                     default:
                         break;
                     }
-                    currBind->buffer_length = sizeof(MYSQL_TIME);
+                    currBind->buffer_length = sizeof(QT_MYSQL_TIME);
                     currBind->length = 0;
                     break; }
                 case QMetaType::UInt:
@@ -994,7 +977,7 @@ bool QMYSQLResult::exec()
                     stringVector.append(ba);
                     currBind->buffer_type = MYSQL_TYPE_STRING;
                     currBind->buffer = const_cast<char *>(ba.constData());
-                    currBind->buffer_length = ba.length();
+                    currBind->buffer_length = ba.size();
                     break; }
             }
         }
@@ -1164,19 +1147,19 @@ bool QMYSQLDriver::hasFeature(DriverFeature f) const
 
 static void setOptionFlag(uint &optionFlags, const QString &opt)
 {
-    if (opt == QLatin1String("CLIENT_COMPRESS"))
+    if (opt == "CLIENT_COMPRESS"_L1)
         optionFlags |= CLIENT_COMPRESS;
-    else if (opt == QLatin1String("CLIENT_FOUND_ROWS"))
+    else if (opt == "CLIENT_FOUND_ROWS"_L1)
         optionFlags |= CLIENT_FOUND_ROWS;
-    else if (opt == QLatin1String("CLIENT_IGNORE_SPACE"))
+    else if (opt == "CLIENT_IGNORE_SPACE"_L1)
         optionFlags |= CLIENT_IGNORE_SPACE;
-    else if (opt == QLatin1String("CLIENT_INTERACTIVE"))
+    else if (opt == "CLIENT_INTERACTIVE"_L1)
         optionFlags |= CLIENT_INTERACTIVE;
-    else if (opt == QLatin1String("CLIENT_NO_SCHEMA"))
+    else if (opt == "CLIENT_NO_SCHEMA"_L1)
         optionFlags |= CLIENT_NO_SCHEMA;
-    else if (opt == QLatin1String("CLIENT_ODBC"))
+    else if (opt == "CLIENT_ODBC"_L1)
         optionFlags |= CLIENT_ODBC;
-    else if (opt == QLatin1String("CLIENT_SSL"))
+    else if (opt == "CLIENT_SSL"_L1)
         qWarning("QMYSQLDriver: SSL_KEY, SSL_CERT and SSL_CA should be used instead of CLIENT_SSL.");
     else
         qWarning("QMYSQLDriver::open: Unknown connect option '%s'", opt.toLocal8Bit().constData());
@@ -1199,7 +1182,7 @@ bool QMYSQLDriver::open(const QString& db,
        stored procedure call will fail.
     */
     unsigned int optionFlags = CLIENT_MULTI_STATEMENTS;
-    const QStringList opts(connOpts.split(QLatin1Char(';'), Qt::SkipEmptyParts));
+    const QStringList opts(connOpts.split(u';', Qt::SkipEmptyParts));
     QString unixSocket;
     QString sslCert;
     QString sslCA;
@@ -1212,34 +1195,34 @@ bool QMYSQLDriver::open(const QString& db,
     uint writeTimeout = 0;
 
     // extract the real options from the string
-    for (int i = 0; i < opts.count(); ++i) {
+    for (int i = 0; i < opts.size(); ++i) {
         QString tmp(opts.at(i).simplified());
-        int idx;
-        if ((idx = tmp.indexOf(QLatin1Char('='))) != -1) {
+        qsizetype idx;
+        if ((idx = tmp.indexOf(u'=')) != -1) {
             QString val = tmp.mid(idx + 1).simplified();
             QString opt = tmp.left(idx).simplified();
-            if (opt == QLatin1String("UNIX_SOCKET"))
+            if (opt == "UNIX_SOCKET"_L1)
                 unixSocket = val;
-            else if (opt == QLatin1String("MYSQL_OPT_RECONNECT")) {
-                if (val == QLatin1String("TRUE") || val == QLatin1String("1") || val.isEmpty())
+            else if (opt == "MYSQL_OPT_RECONNECT"_L1) {
+                if (val == "TRUE"_L1 || val == "1"_L1 || val.isEmpty())
                     reconnect = true;
-            } else if (opt == QLatin1String("MYSQL_OPT_CONNECT_TIMEOUT"))
+            } else if (opt == "MYSQL_OPT_CONNECT_TIMEOUT"_L1)
                 connectTimeout = val.toInt();
-            else if (opt == QLatin1String("MYSQL_OPT_READ_TIMEOUT"))
+            else if (opt == "MYSQL_OPT_READ_TIMEOUT"_L1)
                 readTimeout = val.toInt();
-            else if (opt == QLatin1String("MYSQL_OPT_WRITE_TIMEOUT"))
+            else if (opt == "MYSQL_OPT_WRITE_TIMEOUT"_L1)
                 writeTimeout = val.toInt();
-            else if (opt == QLatin1String("SSL_KEY"))
+            else if (opt == "SSL_KEY"_L1)
                 sslKey = val;
-            else if (opt == QLatin1String("SSL_CERT"))
+            else if (opt == "SSL_CERT"_L1)
                 sslCert = val;
-            else if (opt == QLatin1String("SSL_CA"))
+            else if (opt == "SSL_CA"_L1)
                 sslCA = val;
-            else if (opt == QLatin1String("SSL_CAPATH"))
+            else if (opt == "SSL_CAPATH"_L1)
                 sslCAPath = val;
-            else if (opt == QLatin1String("SSL_CIPHER"))
+            else if (opt == "SSL_CIPHER"_L1)
                 sslCipher = val;
-            else if (val == QLatin1String("TRUE") || val == QLatin1String("1"))
+            else if (val == "TRUE"_L1 || val == "1"_L1)
                 setOptionFlag(optionFlags, tmp.left(idx).simplified());
             else
                 qWarning("QMYSQLDriver::open: Illegal connect option value '%s'",
@@ -1370,14 +1353,14 @@ QStringList QMYSQLDriver::tables(QSql::TableType type) const
     QStringList tl;
     QSqlQuery q(createResult());
     if (type & QSql::Tables) {
-        QString sql = QLatin1String("select table_name from information_schema.tables where table_schema = '") + QLatin1String(d->mysql->db) + QLatin1String("' and table_type = 'BASE TABLE'");
+        QString sql = "select table_name from information_schema.tables where table_schema = '"_L1 + QLatin1StringView(d->mysql->db) + "' and table_type = 'BASE TABLE'"_L1;
         q.exec(sql);
 
         while (q.next())
             tl.append(q.value(0).toString());
     }
     if (type & QSql::Views) {
-        QString sql = QLatin1String("select table_name from information_schema.tables where table_schema = '") + QLatin1String(d->mysql->db) + QLatin1String("' and table_type = 'VIEW'");
+        QString sql = "select table_name from information_schema.tables where table_schema = '"_L1 + QLatin1StringView(d->mysql->db) + "' and table_type = 'VIEW'"_L1;
         q.exec(sql);
 
         while (q.next())
@@ -1393,11 +1376,11 @@ QSqlIndex QMYSQLDriver::primaryIndex(const QString& tablename) const
         return idx;
 
     QSqlQuery i(createResult());
-    QString stmt(QLatin1String("show index from %1;"));
+    QString stmt("show index from %1;"_L1);
     QSqlRecord fil = record(tablename);
     i.exec(stmt.arg(escapeIdentifier(tablename, QSqlDriver::TableName)));
     while (i.isActive() && i.next()) {
-        if (i.value(2).toString() == QLatin1String("PRIMARY")) {
+        if (i.value(2).toString() == "PRIMARY"_L1) {
             idx.append(fil.field(i.value(4).toString()));
             idx.setCursorName(i.value(0).toString());
             idx.setName(i.value(2).toString());
@@ -1494,7 +1477,7 @@ QString QMYSQLDriver::formatValue(const QSqlField &field, bool trimStrings) cons
         case QMetaType::QString:
             // Escape '\' characters
             r = QSqlDriver::formatValue(field, trimStrings);
-            r.replace(QLatin1String("\\"), QLatin1String("\\\\"));
+            r.replace("\\"_L1, "\\\\"_L1);
             break;
         case QMetaType::QByteArray:
             if (isOpen()) {
@@ -1503,7 +1486,7 @@ QString QMYSQLDriver::formatValue(const QSqlField &field, bool trimStrings) cons
                 QVarLengthArray<char, 512> buffer(ba.size() * 2 + 1);
                 auto escapedSize = mysql_real_escape_string(d->mysql, buffer.data(), ba.data(), ba.size());
                 r.reserve(escapedSize + 3);
-                r = QLatin1Char('\'') + QString::fromUtf8(buffer) + QLatin1Char('\'');
+                r = u'\'' + QString::fromUtf8(buffer.data(), escapedSize) + u'\'';
                 break;
             } else {
                 qWarning("QMYSQLDriver::formatValue: Database not open");
@@ -1516,11 +1499,11 @@ QString QMYSQLDriver::formatValue(const QSqlField &field, bool trimStrings) cons
                 // it's because the MySQL server is too old for prepared queries
                 // in the first place, so it won't understand timezones either.
                 // Besides, MYSQL_TIME does not support timezones, so match it.
-                r = QLatin1Char('\'') +
+                r = u'\'' +
                         dt.date().toString(Qt::ISODate) +
-                        QLatin1Char('T') +
+                        u'T' +
                         dt.time().toString(Qt::ISODate) +
-                        QLatin1Char('\'');
+                        u'\'';
             }
             break;
         default:
@@ -1533,9 +1516,9 @@ QString QMYSQLDriver::formatValue(const QSqlField &field, bool trimStrings) cons
 QString QMYSQLDriver::escapeIdentifier(const QString &identifier, IdentifierType) const
 {
     QString res = identifier;
-    if (!identifier.isEmpty() && !identifier.startsWith(QLatin1Char('`')) && !identifier.endsWith(QLatin1Char('`')) ) {
-        res.prepend(QLatin1Char('`')).append(QLatin1Char('`'));
-        res.replace(QLatin1Char('.'), QLatin1String("`.`"));
+    if (!identifier.isEmpty() && !identifier.startsWith(u'`') && !identifier.endsWith(u'`') ) {
+        res.prepend(u'`').append(u'`');
+        res.replace(u'.', "`.`"_L1);
     }
     return res;
 }
@@ -1544,8 +1527,8 @@ bool QMYSQLDriver::isIdentifierEscaped(const QString &identifier, IdentifierType
 {
     Q_UNUSED(type);
     return identifier.size() > 2
-        && identifier.startsWith(QLatin1Char('`')) //left delimited
-        && identifier.endsWith(QLatin1Char('`')); //right delimited
+        && identifier.startsWith(u'`') //left delimited
+        && identifier.endsWith(u'`'); //right delimited
 }
 
 QT_END_NAMESPACE
