@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include <qmimedatabase.h>
 
@@ -43,7 +18,14 @@
 #include <QtCore/QTextStream>
 #include <QtConcurrent/QtConcurrentRun>
 
-#include <QtTest/QtTest>
+#include <QTest>
+#include <QBuffer>
+#include <QTemporaryFile>
+#if QT_CONFIG(process)
+#include <QProcess>
+#endif
+
+using namespace Qt::StringLiterals;
 
 static const char *const additionalMimeFiles[] = {
     "yast2-metapackage-handler-mimetypes.xml",
@@ -70,15 +52,15 @@ static inline QString testSuiteWarning()
 
     QString result;
     QTextStream str(&result);
-    str << "\nCannot find the shared-mime-info test suite\nstarting from: "
+    str << "\nCannot find the shared-mime-info test suite\nin the parent of: "
         << QDir::toNativeSeparators(QDir::currentPath()) << "\n"
            "cd " << QDir::toNativeSeparators(QStringLiteral("tests/auto/corelib/mimetypes/qmimedatabase")) << "\n"
-           "wget http://cgit.freedesktop.org/xdg/shared-mime-info/snapshot/Release-1-10.zip\n"
-           "unzip Release-1-10.zip\n";
+           "wget https://gitlab.freedesktop.org/xdg/shared-mime-info/-/archive/2.2/shared-mime-info-2.2.zip\n"
+           "unzip shared-mime-info-2.2.zip\n";
 #ifdef Q_OS_WIN
-    str << "mkdir testfiles\nxcopy /s Release-1-10 s-m-i\n";
+    str << "mkdir testfiles\nxcopy /s shared-mime-info-2.2 s-m-i\n";
 #else
-    str << "ln -s Release-1-10 s-m-i\n";
+    str << "ln -s shared-mime-info-2.2 s-m-i\n";
 #endif
     return result;
 }
@@ -154,7 +136,7 @@ void tst_QMimeDatabase::initTestCase()
     QVERIFY2(copyResourceFile(xmlFileName, xmlTargetFileName, &errorMessage), qPrintable(errorMessage));
 #endif
 
-    m_testSuite = QFINDTESTDATA("s-m-i/tests");
+    m_testSuite = QFINDTESTDATA("../s-m-i/tests/mime-detection");
     if (m_testSuite.isEmpty())
         qWarning("%s", qPrintable(testSuiteWarning()));
 
@@ -265,6 +247,7 @@ void tst_QMimeDatabase::mimeTypeForFileName_data()
     // fdo bug 15436, needs shared-mime-info >= 0.40 (and this tests the globs2-parsing code).
     QTest::newRow("glob that ends with *, also matches *.pdf. *.pdf has higher weight") << "README.pdf" << "application/pdf";
     QTest::newRow("directory") << "/" << "inode/directory";
+    QTest::newRow("resource-directory") << ":/files/" << "inode/directory";
     QTest::newRow("doesn't exist, no extension") << "IDontExist" << "application/octet-stream";
     QTest::newRow("doesn't exist but has known extension") << "IDontExist.txt" << "text/plain";
     QTest::newRow("empty") << "" << "application/octet-stream";
@@ -297,7 +280,7 @@ void tst_QMimeDatabase::mimeTypeForFileName()
         QVERIFY(mimes.isEmpty());
     } else {
         QVERIFY2(!mimes.isEmpty(), msgMimeTypeForFileNameFailed(mimes, expectedMimeType).constData());
-        QVERIFY2(mimes.count() == 1, msgMimeTypeForFileNameFailed(mimes, expectedMimeType).constData());
+        QVERIFY2(mimes.size() == 1, msgMimeTypeForFileNameFailed(mimes, expectedMimeType).constData());
         QCOMPARE(mimes.first().name(), expectedMimeType);
     }
 }
@@ -348,7 +331,7 @@ void tst_QMimeDatabase::inheritance()
 
     const QMimeType directory = db.mimeTypeForName(QString::fromLatin1("inode/directory"));
     QVERIFY(directory.isValid());
-    QCOMPARE(directory.parentMimeTypes().count(), 0);
+    QCOMPARE(directory.parentMimeTypes().size(), 0);
     QVERIFY(!directory.inherits(QLatin1String("application/octet-stream")));
 
     // Check that text/x-patch knows that it inherits from text/plain (it says so explicitly)
@@ -369,7 +352,7 @@ void tst_QMimeDatabase::inheritance()
     const QStringList shellParents = shellscript.parentMimeTypes();
     QVERIFY(shellParents.contains(QLatin1String("text/plain")));
     QVERIFY(shellParents.contains(QLatin1String("application/x-executable")));
-    QCOMPARE(shellParents.count(), 2); // only the above two
+    QCOMPARE(shellParents.size(), 2); // only the above two
     const QStringList allShellAncestors = shellscript.allAncestors();
     QVERIFY(allShellAncestors.contains(QLatin1String("text/plain")));
     QVERIFY(allShellAncestors.contains(QLatin1String("application/x-executable")));
@@ -427,7 +410,7 @@ void tst_QMimeDatabase::listAliases()
     QFETCH(QString, inputMime);
     QFETCH(QString, expectedAliases);
     QMimeDatabase db;
-    QStringList expectedAliasesList = expectedAliases.split(',', QString::SkipEmptyParts);
+    QStringList expectedAliasesList = expectedAliases.split(',', Qt::SkipEmptyParts);
     expectedAliasesList.sort();
     QMimeType mime = db.mimeTypeForName(inputMime);
     QVERIFY(mime.isValid());
@@ -517,6 +500,42 @@ void tst_QMimeDatabase::mimeTypeForFileWithContent()
         QCOMPARE(mime.name(), QString::fromLatin1("application/smil+xml"));
     }
 
+    // Test what happens with Qt resources (file engines in general)
+    {
+        QFile rccFile(":/files/test.txt");
+
+        mime = db.mimeTypeForFile(rccFile.fileName());
+        QCOMPARE(mime.name(), "text/plain"_L1);
+
+        QVERIFY(rccFile.open(QIODevice::ReadOnly));
+        mime = db.mimeTypeForData(&rccFile);
+        QCOMPARE(mime.name(), "text/x-qml"_L1);
+        QVERIFY(rccFile.isOpen());
+
+        mime = db.mimeTypeForFile(rccFile.fileName(), QMimeDatabase::MatchContent);
+        QCOMPARE(mime.name(), "text/x-qml"_L1);
+    }
+
+    // Directories
+    {
+        mime = db.mimeTypeForFile("/");
+        QCOMPARE(mime.name(), "inode/directory"_L1);
+
+        QString dirName = QDir::tempPath();
+        if (!dirName.endsWith(u'/'))
+            dirName += u'/';
+        mime = db.mimeTypeForFile(dirName);
+        QCOMPARE(mime.name(), "inode/directory"_L1);
+
+        while (dirName.endsWith(u'/'))
+            dirName.chop(1);
+        mime = db.mimeTypeForFile(dirName);
+        QCOMPARE(mime.name(), "inode/directory"_L1);
+
+        mime = db.mimeTypeForFile(":/files");
+        QCOMPARE(mime.name(), "inode/directory"_L1);
+    }
+
     // Test what happens with an incorrect path
     mime = db.mimeTypeForFile(QString::fromLatin1("file:///etc/passwd" /* incorrect code, use a path instead */));
     QVERIFY(mime.isDefault());
@@ -611,7 +630,7 @@ void tst_QMimeDatabase::allMimeTypes()
     QVERIFY(!lst.isEmpty());
 
     // Hardcoding this is the only way to check both providers find the same number of mimetypes.
-    QCOMPARE(lst.count(), 779);
+    QCOMPARE(lst.size(), 851);
 
     foreach (const QMimeType &mime, lst) {
         const QString name = mime.name();
@@ -631,10 +650,9 @@ void tst_QMimeDatabase::suffixes_data()
 
     QTest::newRow("mimetype with a single pattern") << "application/pdf" << "*.pdf" << "pdf";
     QTest::newRow("mimetype with multiple patterns") << "application/x-kpresenter" << "*.kpr;*.kpt" << "kpr";
-    QTest::newRow("jpeg") << "image/jpeg" << "*.jpe;*.jpg;*.jpeg" << "jpeg";
-    //if (KMimeType::sharedMimeInfoVersion() > KDE_MAKE_VERSION(0, 60, 0)) {
-        QTest::newRow("mimetype with many patterns") << "application/vnd.wordperfect" << "*.wp;*.wp4;*.wp5;*.wp6;*.wpd;*.wpp" << "wp";
-    //}
+    // The preferred suffix for image/jpeg is *.jpg, as per https://bugs.kde.org/show_bug.cgi?id=176737
+    QTest::newRow("jpeg") << "image/jpeg" << "*.jpe;*.jpg;*.jpeg" << "jpg";
+    QTest::newRow("mimetype with many patterns") << "application/vnd.wordperfect" << "*.wp;*.wp4;*.wp5;*.wp6;*.wpd;*.wpp" << "wp";
     QTest::newRow("oasis text mimetype") << "application/vnd.oasis.opendocument.text" << "*.odt" << "odt";
     QTest::newRow("oasis presentation mimetype") << "application/vnd.oasis.opendocument.presentation" << "*.odp" << "odp";
     QTest::newRow("mimetype with multiple patterns") << "text/plain" << "*.asc;*.txt;*,v" << "txt";
@@ -667,11 +685,14 @@ void tst_QMimeDatabase::knownSuffix()
     QCOMPARE(db.suffixForFileName(QString::fromLatin1("foo.bz2")), QString::fromLatin1("bz2"));
     QCOMPARE(db.suffixForFileName(QString::fromLatin1("foo.bar.bz2")), QString::fromLatin1("bz2"));
     QCOMPARE(db.suffixForFileName(QString::fromLatin1("foo.tar.bz2")), QString::fromLatin1("tar.bz2"));
+    QCOMPARE(db.suffixForFileName(QString::fromLatin1("foo.TAR")), QString::fromLatin1("TAR")); // preserve case
+    QCOMPARE(db.suffixForFileName(QString::fromLatin1("foo.flatpakrepo")), QString::fromLatin1("flatpakrepo"));
+    QCOMPARE(db.suffixForFileName(QString::fromLatin1("foo.anim2")), QString()); // the glob is anim[0-9], no way to extract the extension without expensive regexp capturing
 }
 
 void tst_QMimeDatabase::symlinkToFifo() // QTBUG-48529
 {
-#ifdef Q_OS_UNIX
+#if defined(Q_OS_UNIX) && !defined(Q_OS_INTEGRITY)
     QTemporaryDir tempDir;
     QVERIFY(tempDir.isValid());
     const QString dir = tempDir.path();
@@ -715,7 +736,7 @@ void tst_QMimeDatabase::findByFileName_data()
             continue;
 
         QString string = QString::fromLatin1(line.constData(), len - 1).trimmed();
-        QStringList list = string.split(QLatin1Char(' '), QString::SkipEmptyParts);
+        QStringList list = string.split(QLatin1Char(' '), Qt::SkipEmptyParts);
         QVERIFY(list.size() >= 2);
 
         QString filePath = list.at(0);
@@ -759,7 +780,7 @@ void tst_QMimeDatabase::findByFileName()
     //qDebug() << Q_FUNC_INFO << "mimeTypeForFile() returned" << resultMimeTypeName;
 
     const bool failed = resultMimeTypeName != mimeTypeName;
-    const bool shouldFail = (xFail.length() >= 1 && xFail.at(0) == QLatin1Char('x'));
+    const bool shouldFail = (xFail.size() >= 1 && xFail.at(0) == QLatin1Char('x'));
     if (shouldFail != failed) {
         // Results are ambiguous when multiple MIME types have the same glob
         // -> accept the current result if the found MIME type actually
@@ -784,6 +805,9 @@ void tst_QMimeDatabase::findByFileName()
     // Test QFileInfo overload
     const QMimeType mimeForFileInfo = database.mimeTypeForFile(QFileInfo(filePath), QMimeDatabase::MatchExtension);
     QCOMPARE(mimeForFileInfo.name(), resultMimeTypeName);
+
+    const QString suffix = database.suffixForFileName(filePath);
+    QVERIFY2(filePath.endsWith(suffix), qPrintable(filePath + " does not end with " + suffix));
 }
 
 void tst_QMimeDatabase::findByData_data()
@@ -803,7 +827,7 @@ void tst_QMimeDatabase::findByData()
     QByteArray data = f.read(16384);
 
     const QString resultMimeTypeName = database.mimeTypeForData(data).name();
-    if (xFail.length() >= 2 && xFail.at(1) == QLatin1Char('x')) {
+    if (xFail.size() >= 2 && xFail.at(1) == QLatin1Char('x')) {
         // Expected to fail
         QVERIFY2(resultMimeTypeName != mimeTypeName, qPrintable(resultMimeTypeName));
     } else {
@@ -834,7 +858,7 @@ void tst_QMimeDatabase::findByFile()
     QMimeDatabase database;
     const QString resultMimeTypeName = database.mimeTypeForFile(filePath).name();
     //qDebug() << Q_FUNC_INFO << filePath << "->" << resultMimeTypeName;
-    if (xFail.length() >= 3 && xFail.at(2) == QLatin1Char('x')) {
+    if (xFail.size() >= 3 && xFail.at(2) == QLatin1Char('x')) {
         // Expected to fail
         QVERIFY2(resultMimeTypeName != mimeTypeName, qPrintable(resultMimeTypeName));
     } else {
@@ -852,14 +876,14 @@ void tst_QMimeDatabase::fromThreads()
     QThreadPool tp;
     tp.setMaxThreadCount(20);
     // Note that data-based tests cannot be used here (QTest::fetchData asserts).
-    QtConcurrent::run(&tp, this, &tst_QMimeDatabase::mimeTypeForName);
-    QtConcurrent::run(&tp, this, &tst_QMimeDatabase::aliases);
-    QtConcurrent::run(&tp, this, &tst_QMimeDatabase::allMimeTypes);
-    QtConcurrent::run(&tp, this, &tst_QMimeDatabase::icons);
-    QtConcurrent::run(&tp, this, &tst_QMimeDatabase::inheritance);
-    QtConcurrent::run(&tp, this, &tst_QMimeDatabase::knownSuffix);
-    QtConcurrent::run(&tp, this, &tst_QMimeDatabase::mimeTypeForFileWithContent);
-    QtConcurrent::run(&tp, this, &tst_QMimeDatabase::allMimeTypes); // a second time
+    Q_UNUSED(QtConcurrent::run(&tp, &tst_QMimeDatabase::mimeTypeForName, this));
+    Q_UNUSED(QtConcurrent::run(&tp, &tst_QMimeDatabase::aliases, this));
+    Q_UNUSED(QtConcurrent::run(&tp, &tst_QMimeDatabase::allMimeTypes, this));
+    Q_UNUSED(QtConcurrent::run(&tp, &tst_QMimeDatabase::icons, this));
+    Q_UNUSED(QtConcurrent::run(&tp, &tst_QMimeDatabase::inheritance, this));
+    Q_UNUSED(QtConcurrent::run(&tp, &tst_QMimeDatabase::knownSuffix, this));
+    Q_UNUSED(QtConcurrent::run(&tp, &tst_QMimeDatabase::mimeTypeForFileWithContent, this));
+    Q_UNUSED(QtConcurrent::run(&tp, &tst_QMimeDatabase::allMimeTypes, this)); // a second time
     QVERIFY(tp.waitForDone(60000));
 }
 
@@ -877,7 +901,7 @@ static bool runUpdateMimeDatabase(const QString &path) // TODO make it a QMimeDa
         qWarning("%s does not exist.", qPrintable(umdCommand));
         return false;
     }
-
+#if QT_CONFIG(process)
     QElapsedTimer timer;
     QProcess proc;
     proc.setProcessChannelMode(QProcess::MergedChannels); // silence output
@@ -892,6 +916,7 @@ static bool runUpdateMimeDatabase(const QString &path) // TODO make it a QMimeDa
     const bool success = proc.waitForFinished(UpdateMimeDatabaseTimeout);
     qDebug().noquote() << "runUpdateMimeDatabase: done,"
         << success << timer.elapsed() << "ms";
+#endif
     return true;
 }
 
@@ -979,11 +1004,10 @@ void tst_QMimeDatabase::installNewGlobalMimeType()
     QCOMPARE(db.mimeTypeForFile(qmlTestFile).name(),
              QString::fromLatin1("text/x-qml"));
 
-    // ensure we can access the empty glob list
     {
         QMimeType objcsrc = db.mimeTypeForName(QStringLiteral("text/x-objcsrc"));
         QVERIFY(objcsrc.isValid());
-        qDebug() << objcsrc.globPatterns();
+        QCOMPARE(objcsrc.globPatterns(), QStringList());
     }
 
     const QString fooTestFile = QLatin1String(RESOURCE_PREFIX "magic-and-hierarchy.foo");
@@ -1060,6 +1084,12 @@ void tst_QMimeDatabase::installNewLocalMimeType()
     QCOMPARE(db.mimeTypeForName(QLatin1String("text/x-SuSE-ymu")).comment(), QString("URL of a YaST Meta Package"));
     checkHasMimeType("text/x-suse-ymp");
 
+    { // QTBUG-85436
+        QMimeType objcsrc = db.mimeTypeForName(QStringLiteral("text/x-objcsrc"));
+        QVERIFY(objcsrc.isValid());
+        QCOMPARE(objcsrc.globPatterns(), QStringList());
+    }
+
     // Test that a double-definition of a mimetype doesn't lead to sniffing ("conflicting globs").
     const QString qmlTestFile = QLatin1String(RESOURCE_PREFIX "test.qml");
     QVERIFY2(!qmlTestFile.isEmpty(),
