@@ -1,40 +1,16 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
-
-#include <QtTest/QtTest>
 #include <QtNetwork/QtNetwork>
+#include <QTest>
+#include <QTestEventLoop>
 #include <qnetworkdiskcache.h>
 #include <qrandom.h>
 
 #include <algorithm>
 
 #define EXAMPLE_URL "http://user:pass@localhost:4/#foo"
+#define EXAMPLE_URL2 "http://user:pass@localhost:4/bar"
 //cached objects are organized into these many subdirs
 #define NUM_SUBDIRECTORIES 16
 
@@ -141,7 +117,7 @@ class SubQNetworkDiskCache : public QNetworkDiskCache
 public:
     ~SubQNetworkDiskCache()
     {
-        if (!cacheDirectory().isEmpty())
+        if (!cacheDirectory().isEmpty() && clearOnDestruction)
             clear();
     }
 
@@ -170,6 +146,11 @@ public:
         d->write("Hello World!");
         insert(d);
     }
+
+    void setClearCacheOnDestruction(bool value) { clearOnDestruction = value; }
+
+private:
+    bool clearOnDestruction = true;
 };
 
 tst_QNetworkDiskCache::tst_QNetworkDiskCache()
@@ -241,17 +222,39 @@ void tst_QNetworkDiskCache::prepare()
 // public qint64 cacheSize() const
 void tst_QNetworkDiskCache::cacheSize()
 {
+    qint64 cacheSize = 0;
+    {
+        SubQNetworkDiskCache cache;
+        cache.setCacheDirectory(tempDir.path());
+        QCOMPARE(cache.cacheSize(), qint64(0));
+
+        {
+            QUrl url(EXAMPLE_URL);
+            QNetworkCacheMetaData metaData;
+            metaData.setUrl(url);
+            QIODevice *d = cache.prepare(metaData);
+            cache.insert(d);
+            cacheSize = cache.cacheSize();
+            QVERIFY(cacheSize > qint64(0));
+        }
+        // Add a second item, some difference in behavior when the cache is not empty
+        {
+            QUrl url(EXAMPLE_URL2);
+            QNetworkCacheMetaData metaData;
+            metaData.setUrl(url);
+            QIODevice *d = cache.prepare(metaData);
+            cache.insert(d);
+            QVERIFY(cache.cacheSize() > cacheSize);
+            cacheSize = cache.cacheSize();
+        }
+
+        // Don't clear the cache on destruction so we can re-open the cache and test its size.
+        cache.setClearCacheOnDestruction(false);
+    }
+
     SubQNetworkDiskCache cache;
     cache.setCacheDirectory(tempDir.path());
-    QCOMPARE(cache.cacheSize(), qint64(0));
-
-    QUrl url(EXAMPLE_URL);
-    QNetworkCacheMetaData metaData;
-    metaData.setUrl(url);
-    QIODevice *d = cache.prepare(metaData);
-    cache.insert(d);
-    QVERIFY(cache.cacheSize() > qint64(0));
-
+    QCOMPARE(cache.cacheSize(), cacheSize);
     cache.clear();
     QCOMPARE(cache.cacheSize(), qint64(0));
 }
@@ -275,17 +278,17 @@ void tst_QNetworkDiskCache::clear()
     QVERIFY(cache.cacheSize() > qint64(0));
 
     QString cacheDirectory = cache.cacheDirectory();
-    QCOMPARE(countFiles(cacheDirectory).count(), NUM_SUBDIRECTORIES + 3);
+    QCOMPARE(countFiles(cacheDirectory).size(), NUM_SUBDIRECTORIES + 3);
     cache.clear();
-    QCOMPARE(countFiles(cacheDirectory).count(), NUM_SUBDIRECTORIES + 2);
+    QCOMPARE(countFiles(cacheDirectory).size(), NUM_SUBDIRECTORIES + 2);
 
     // don't delete files that it didn't create
     QTemporaryFile file(cacheDirectory + "/XXXXXX");
     if (file.open()) {
         file.fileName();    // make sure it exists with a name
-        QCOMPARE(countFiles(cacheDirectory).count(), NUM_SUBDIRECTORIES + 3);
+        QCOMPARE(countFiles(cacheDirectory).size(), NUM_SUBDIRECTORIES + 3);
         cache.clear();
-        QCOMPARE(countFiles(cacheDirectory).count(), NUM_SUBDIRECTORIES + 3);
+        QCOMPARE(countFiles(cacheDirectory).size(), NUM_SUBDIRECTORIES + 3);
     }
 }
 
@@ -352,9 +355,9 @@ void tst_QNetworkDiskCache::remove()
     QUrl url(EXAMPLE_URL);
     cache.setupWithOne(tempDir.path(), url);
     QString cacheDirectory = cache.cacheDirectory();
-    QCOMPARE(countFiles(cacheDirectory).count(), NUM_SUBDIRECTORIES + 3);
+    QCOMPARE(countFiles(cacheDirectory).size(), NUM_SUBDIRECTORIES + 3);
     cache.remove(url);
-    QCOMPARE(countFiles(cacheDirectory).count(), NUM_SUBDIRECTORIES + 2);
+    QCOMPARE(countFiles(cacheDirectory).size(), NUM_SUBDIRECTORIES + 2);
 }
 
 void tst_QNetworkDiskCache::accessAfterRemove() // QTBUG-17400
@@ -474,7 +477,7 @@ void tst_QNetworkDiskCache::fileMetaData()
 
     QString cacheDirectory = cache.cacheDirectory();
     QStringList list = countFiles(cacheDirectory);
-    QCOMPARE(list.count(), NUM_SUBDIRECTORIES + 3);
+    QCOMPARE(list.size(), NUM_SUBDIRECTORIES + 3);
     foreach(QString fileName, list) {
         QFileInfo info(fileName);
         if (info.isFile()) {
@@ -528,7 +531,7 @@ void tst_QNetworkDiskCache::expire()
         }
     }
     std::sort(cacheList.begin(), cacheList.end());
-    for (int i = 0; i < cacheList.count(); ++i) {
+    for (int i = 0; i < cacheList.size(); ++i) {
         QString fileName = cacheList[i];
         QCOMPARE(fileName, QLatin1String("http://localhost:4/") + QString::number(i + 6));
     }
@@ -567,7 +570,7 @@ void tst_QNetworkDiskCache::oldCacheVersionFile()
         QVERIFY(!QFile::exists(name));
     } else {
         QStringList files = countFiles(cache.cacheDirectory());
-        QCOMPARE(files.count(), NUM_SUBDIRECTORIES + 3);
+        QCOMPARE(files.size(), NUM_SUBDIRECTORIES + 3);
         // find the file
         QString cacheFile;
         foreach (QString file, files) {
@@ -733,7 +736,7 @@ public:
                 if (d) {
                     QByteArray x = d->readAll();
                     if (x != longString && x != longString2) {
-                        qDebug() << x.length() << QString(x);
+                        qDebug() << x.size() << QString(x);
                         gotMetaData = cache.metaData(url);
                         qDebug() << (gotMetaData.url().toString())
                          << gotMetaData.lastModified()
